@@ -2,6 +2,9 @@
 
 #include <openssl/cms.h>
 #include <glib.h>
+#include <gio/gio.h>
+
+#include "manifest.h"
 
 #define R_BUNDLE_ERROR r_bundle_error_quark()
 GQuark r_bundle_error_quark(void);
@@ -10,15 +13,25 @@ typedef enum {
 	R_BUNDLE_ERROR_SIGNATURE,
 	R_BUNDLE_ERROR_KEYRING,
 	R_BUNDLE_ERROR_IDENTIFIER,
+	R_BUNDLE_ERROR_UNSAFE,
+	R_BUNDLE_ERROR_PAYLOAD,
+	R_BUNDLE_ERROR_FORMAT,
 } RBundleError;
 
 typedef struct {
 	gchar *path;
 	gchar *origpath;
 	gchar *storepath;
+	GInputStream *stream;
 	goffset size;
 	GBytes *sigdata;
 	gchar *mount_point;
+	RaucManifest *manifest;
+	gboolean verification_disabled;
+	gboolean signature_verified;
+	gboolean payload_verified;
+	gboolean exclusive_verified;
+	gchar *exclusive_check_error;
 	STACK_OF(X509) *verified_chain;
 } RaucBundle;
 
@@ -31,7 +44,8 @@ typedef struct {
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError **error);
+gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Check a bundle.
@@ -48,7 +62,28 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean verify, GError **error);
+gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean verify, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Check the bundle payload, if needed and possible.
+ *
+ * For verity bundles, this checks the full squashfs against the dm-verity hash
+ * tree and the root hash against the one contained in the manifest.
+ * This is mainly useful for when extracting the bundle contents without
+ * mounting.
+ * It will set the payload_verified struct member to remember that the payload
+ * has been verified.
+ *
+ * @param bundle the RaucBundle struct to verify
+ *               This will contain all bundle information obtained by
+ *               check_bundle
+ * @param error Return location for a GError
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+gboolean check_bundle_payload(RaucBundle *bundle, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Resign a bundle.
@@ -65,7 +100,8 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean resign_bundle(RaucBundle *bundle, const gchar *outpath, GError **error);
+gboolean resign_bundle(RaucBundle *bundle, const gchar *outpath, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Extract a bundle.
@@ -81,7 +117,8 @@ gboolean resign_bundle(RaucBundle *bundle, const gchar *outpath, GError **error)
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **error);
+gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Extract a single file from a bundle.
@@ -98,7 +135,26 @@ gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **err
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean extract_file_from_bundle(RaucBundle *bundle, const gchar *outputdir, const gchar *file, GError **error);
+gboolean extract_file_from_bundle(RaucBundle *bundle, const gchar *outputdir, const gchar *file, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Extract and load the manifest from a bundle.
+ *
+ * This is mainly useful for plain bundles, as the manifest is already contained in
+ * the signature in other cases and available after signature verification.
+ *
+ * Note that check_bundle() must be called prior to this, to obtain a
+ * RaucBundle struct.
+ *
+ * @param bundle RaucBundle struct as returned by check_bundle()
+ * @param manifest return location for extracted manifest
+ * @param error Return location for a GError
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+gboolean load_manifest_from_bundle(RaucBundle *bundle, RaucManifest **manifest, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Create casync bundle.
@@ -107,7 +163,8 @@ gboolean extract_file_from_bundle(RaucBundle *bundle, const gchar *outputdir, co
  * @param outbundle output location for converted casync bundle
  * @param error Return location for a GError
  */
-gboolean create_casync_bundle(RaucBundle *bundle, const gchar *outbundle, GError **error);
+gboolean create_casync_bundle(RaucBundle *bundle, const gchar *outbundle, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Mount a bundle.
@@ -120,7 +177,8 @@ gboolean create_casync_bundle(RaucBundle *bundle, const gchar *outbundle, GError
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean mount_bundle(RaucBundle *bundle, GError **error);
+gboolean mount_bundle(RaucBundle *bundle, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Unmount a bundle.
