@@ -2,10 +2,7 @@
 
 #include <glib.h>
 
-struct mbr_switch_partition {
-	guint64 start;          /* address in bytes */
-	guint64 size;           /* size in bytes */
-};
+#include "update_handler.h"
 
 /**
  * Get the address and size of the inactive boot partition
@@ -21,22 +18,10 @@ struct mbr_switch_partition {
  * @return True if succeeded, False if failed
  */
 gboolean r_mbr_switch_get_inactive_partition(const gchar *device,
-		struct mbr_switch_partition *partition,
+		struct boot_switch_partition *partition,
 		guint64 region_start, guint64 region_size,
-		GError **error);
-
-/**
- * Clear the the memory area defined in dest_partition.
- *
- * @param device dev path (/dev/mmcblkX)
- * @param dest_partition partition to be cleared (start & size) *
- * @param error return location for a GError, or NULL
- *
- * @return True if succeeded, False if failed
- */
-gboolean r_mbr_switch_clear_partition(const gchar *device,
-		const struct mbr_switch_partition *dest_partition,
-		GError **error);
+		GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Set the boot partition in master boot record to point to the
@@ -49,5 +34,6 @@ gboolean r_mbr_switch_clear_partition(const gchar *device,
  * @return True if succeeded, False if failed
  */
 gboolean r_mbr_switch_set_boot_partition(const gchar *device,
-		const struct mbr_switch_partition *partition,
-		GError **error);
+		const struct boot_switch_partition *partition,
+		GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
