@@ -4,6 +4,37 @@
 
 #include "config_file.h"
 
+/**
+ * Wrapper for the mount() system call with a configuration intended for use
+ * with bundles.
+ *
+ * Using the external 'mount' command is not needed in this case, as all options
+ * are fixed.
+ *
+ * @param source source path for mount
+ * @param mountpoint destination path for mount
+ * @param error return location for a GError, or NULL
+ *
+ * @return True if succeeded, False if failed
+ */
+gboolean r_mount_bundle(const gchar *source, const gchar *mountpoint, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Wrapper for the umount() system call with a configuration intended for use
+ * with bundles.
+ *
+ * Using the external 'umount' command is not needed in this case, as all
+ * options are fixed.
+ *
+ * @param mountpoint destination path for mount
+ * @param error return location for a GError, or NULL
+ *
+ * @return True if succeeded, False if failed
+ */
+gboolean r_umount_bundle(const gchar *mountpoint, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Wrapper for calling systems 'mount' command.
  *
@@ -12,26 +43,30 @@
  * @param source source path for mount
  * @param mountpoint destination path for mount
  * @param type type of image to mount (results in -t option)
- * @param size maximum size of image to mount (for loop mounts)
  * @param extra_options additional mount options that will be passed to mount
  *        via `-o` argument
  * @param error return location for a GError, or NULL
  *
  * @return True if succeeded, False if failed
  */
-gboolean r_mount_full(const gchar *source, const gchar *mountpoint, const gchar* type, goffset size, const gchar *extra_options, GError **error);
+gboolean r_mount_full(const gchar *source, const gchar *mountpoint, const gchar* type, const gchar *extra_options, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
- * Loopback mount a file.
+ * Setup a loopback device for a file.
  *
- * @param filename name of file to mount
- * @param mountpoint destination mount point
- * @param size limit accessable size of file, If 0, entire file is used
+ * The size must be > 0, as we always know the exact size of the payload.
+ *
+ * @param fd file descriptor of file to mount
+ * @param loopfd_out file descriptor of the open loop device
+ * @param loopname_out device name of loop device
+ * @param size limit accessable size of file
  * @param error return location for a GError, or NULL
  *
  * @return True if succeeded, False if failed
  */
-gboolean r_mount_loop(const gchar *filename, const gchar *mountpoint, goffset size, GError **error);
+gboolean r_setup_loop(gint fd, gint *loopfd_out, gchar **loopname_out, goffset size, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Unmount a slot or a file.
@@ -41,7 +76,8 @@ gboolean r_mount_loop(const gchar *filename, const gchar *mountpoint, goffset si
  *
  * @return True if succeeded, False if failed
  */
-gboolean r_umount(const gchar *dirdev, GError **error);
+gboolean r_umount(const gchar *dirdev, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Create a mount directory.
@@ -53,7 +89,8 @@ gboolean r_umount(const gchar *dirdev, GError **error);
  *
  * @return A newly allocated string containing the created mount path
  */
-gchar* r_create_mount_point(const gchar *name, GError **error);
+gchar* r_create_mount_point(const gchar *name, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Mount a slot.
@@ -65,7 +102,8 @@ gchar* r_create_mount_point(const gchar *name, GError **error);
  *
  * @return True if succeeded, False if failed
  */
-gboolean r_mount_slot(RaucSlot *slot, GError **error);
+gboolean r_mount_slot(RaucSlot *slot, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Unmount a slot.
