@@ -12,7 +12,8 @@
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean network_init(GError **error);
+gboolean network_init(GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 #else
 static inline gboolean network_init(GError **error)
 {
@@ -20,4 +21,5 @@ static inline gboolean network_init(GError **error)
 }
 #endif
 
-gboolean download_file(const gchar *target, const gchar *url, goffset limit, GError **error);
+gboolean download_file(const gchar *target, const gchar *url, goffset limit, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
