@@ -3,6 +3,8 @@
 #include <openssl/cms.h>
 #include <glib.h>
 
+#include "manifest.h"
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(CMS_ContentInfo, CMS_ContentInfo_free)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(X509_STORE, X509_STORE_free)
 
@@ -38,7 +40,8 @@ typedef enum {
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean signature_init(GError **error);
+gboolean signature_init(GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Prepare an OpenSSL X509_STORE for signature verification.
@@ -52,12 +55,14 @@ gboolean signature_init(GError **error);
  *
  * @return X509_STORE, NULL if failed
  */
-X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **error);
+X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Sign content with provided certificate and private key
  *
  * @param content content that should be signed
+ * @param detached whether the signature should be detached
  * @param certfile certificate file name
  * @param keyfile private key file name
  * @param interfiles NULL-terminated array of intermediate certificate file
@@ -66,7 +71,8 @@ X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **e
  *
  * @return signature bytes, NULL if failed
  */
-GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error);
+GBytes *cms_sign(GBytes *content, gboolean detached, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Sign file with provided certificate and private key
@@ -80,25 +86,70 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
  *
  * @return signature bytes, NULL if failed
  */
-GBytes *cms_sign_file(const gchar *filename, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error);
+GBytes *cms_sign_file(const gchar *filename, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
- * Verify signature for given content.
+ * Sign manifest with provided certificate and private key
  *
- * @param content content to verify against signature
- * @param sig signature used to verify
- * @param store X509 store to use for verification
- * @param cms Return location for the CMS_ContentInfo used for verification
+ * @param manifest manifest that should be serialized and signed
+ * @param certfile certificate file name
+ * @param keyfile private key file name
+ * @param interfiles NULL-terminated array of intermediate certificate file
+ *                   name strings to include in the bundle signature
+ * @param error return location for a GError, or NULL
+ *
+ * @return signature bytes, NULL if failed
+ */
+GBytes *cms_sign_manifest(RaucManifest *manifest, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Check whether the CMS signature is detached or not
+ *
+ * @param sig signature to be checked
+ * @param detached return location for the detached/inline result
  * @param error return location for a GError, or NULL
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GError **error);
+gboolean cms_is_detached(GBytes *sig, gboolean *detached, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
- * Verify signature for given file.
+ * Extract manifest included in a signature without verification.
  *
- * @param filename name of file with content to verify against signature
+ * @param sig signature containing the manifest
+ * @param manifest return location for included manifest
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE if succeeded, FALSE if failed
+ */
+gboolean cms_get_unverified_manifest(GBytes *sig, GBytes **manifest, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Verify detached and inline signatures.
+ *
+ * This function is only used by the tests and internally for cms_verify_fd and
+ * cms_verify_sig.
+ *
+ * @param content content to verify against signature, or NULL (for inline signature)
+ * @param sig signature used to verify
+ * @param store X509 store to use for verification
+ * @param cms Return location for the CMS_ContentInfo used for verification
+ * @param manifest return location for included manifest, or NULL (for detached signature)
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE if succeeded, FALSE if failed
+ */
+gboolean cms_verify_bytes(GBytes *content, GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GBytes **manifest, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Verify detached signature for given file.
+ *
+ * @param fd file descriptor to verify against signature
  * @param sig signature used to verify
  * @param limit size of content to use, 0 if all should be included
  * @param store X509 store to use for verification
@@ -107,7 +158,22 @@ gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_Content
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error);
+gboolean cms_verify_fd(gint fd, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Verify inline signature and return included manifest.
+ *
+ * @param sig signature to verify
+ * @param store X509 store to use for verification
+ * @param cms Return location for the CMS_ContentInfo used for verification
+ * @param manifest return location for included manifest
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE if succeeded, FALSE if failed
+ */
+gboolean cms_verify_sig(GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GBytes **manifest, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Calculates hash for certificate pubkey info.
@@ -119,7 +185,8 @@ gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509
  *
  * @return colon-separated hexadecimal representation of subject key hash
  */
-gchar* get_pubkey_hash(X509 *cert);
+gchar* get_pubkey_hash(X509 *cert)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Calculates all hashes for certificate stacks pubkeys
@@ -128,7 +195,8 @@ gchar* get_pubkey_hash(X509 *cert);
  *
  * @return Array of pointers to string representations of hashes
  */
-gchar** get_pubkey_hashes(STACK_OF(X509) *certs);
+gchar** get_pubkey_hashes(STACK_OF(X509) *certs)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Returns string representation of certificate.
@@ -139,7 +207,8 @@ gchar** get_pubkey_hashes(STACK_OF(X509) *certs);
  * @return allocated string containing default OpenSSL text representation of
  *         signer certificate (first in chain)
  */
-gchar* sigdata_to_string(GBytes *sig, GError **error);
+gchar* sigdata_to_string(GBytes *sig, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Return string representation of certificate chain.
@@ -149,7 +218,8 @@ gchar* sigdata_to_string(GBytes *sig, GError **error);
  * @return allocated string containing text representation of certificate chain
  *         (signer and issuer)
  */
-gchar* print_cert_chain(STACK_OF(X509) *verified_chain);
+gchar* format_cert_chain(STACK_OF(X509) *verified_chain)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Get infos about signer and verification chain.
@@ -164,4 +234,5 @@ gchar* print_cert_chain(STACK_OF(X509) *verified_chain);
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean cms_get_cert_chain(CMS_ContentInfo *cms, X509_STORE *store, STACK_OF(X509) **verified_chain, GError **error);
+gboolean cms_get_cert_chain(CMS_ContentInfo *cms, X509_STORE *store, STACK_OF(X509) **verified_chain, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
