@@ -27,7 +27,7 @@ typedef struct {
 typedef struct _RaucSlot {
 	/** name of the slot. A glib intern string. */
 	const gchar *name;
-	/** user-friendly description of the slot. A glib intern string. */
+	/** user-friendly description of the slot. */
 	gchar *description;
 	/** slot class the slot belongs to. A glib intern string. */
 	const gchar *sclass;
@@ -56,6 +56,8 @@ typedef struct _RaucSlot {
 	SlotState state;
 	gboolean boot_good;
 	struct _RaucSlot *parent;
+	/** the name of the parent as parsed by config (parsing-internal use only) */
+	gchar *parent_name;
 	gchar *mount_point;
 	gchar *ext_mount_point;
 	RaucSlotStatus *status;
@@ -83,7 +85,8 @@ void r_slot_free_status(RaucSlotStatus *slotstatus);
  *
  * @return a RaucSlot pointer or NULL
  */
-RaucSlot *r_slot_find_by_device(GHashTable *slots, const gchar *device);
+RaucSlot *r_slot_find_by_device(GHashTable *slots, const gchar *device)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Finds a slot given its bootname
@@ -93,7 +96,8 @@ RaucSlot *r_slot_find_by_device(GHashTable *slots, const gchar *device);
  *
  * @return a RaucSlot pointer or NULL
  */
-RaucSlot *r_slot_find_by_bootname(GHashTable *slots, const gchar *bootname);
+RaucSlot *r_slot_find_by_bootname(GHashTable *slots, const gchar *bootname)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Get string representation of slot state
@@ -102,7 +106,8 @@ RaucSlot *r_slot_find_by_bootname(GHashTable *slots, const gchar *bootname);
  *
  * @return string representation of slot state
  */
-gchar* r_slot_slotstate_to_str(SlotState slotstate);
+gchar* r_slot_slotstate_to_str(SlotState slotstate)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Get SlotState from string representation.
@@ -111,7 +116,8 @@ gchar* r_slot_slotstate_to_str(SlotState slotstate);
  *
  * @return corresponding SlotState value
  */
-SlotState r_slot_str_to_slotstate(gchar *str);
+SlotState r_slot_str_to_slotstate(gchar *str)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Check if slot type is mountable.
@@ -120,7 +126,8 @@ SlotState r_slot_str_to_slotstate(gchar *str);
  *
  * @return TRUE if mountable, otherwise FALSE
  */
-gboolean r_slot_is_mountable(RaucSlot *slot);
+gboolean r_slot_is_mountable(RaucSlot *slot)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Returns the parent root slot for given slot.
@@ -132,15 +139,16 @@ gboolean r_slot_is_mountable(RaucSlot *slot);
  *
  * @return pointer to RaucSlot
  */
-RaucSlot* r_slot_get_parent_root(RaucSlot *slot);
+RaucSlot* r_slot_get_parent_root(RaucSlot *slot)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Gets all classes that do not have a parent
  *
- * @return newly allocated NULL-teminated string array. Free with g_strfreev
- *         [transfer full]
+ * @return NULL-teminated array of intern strings. Free with g_free().
  */
-gchar** r_slot_get_root_classes(GHashTable *slots);
+gchar** r_slot_get_root_classes(GHashTable *slots)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Test if provided slot list contains slot instance (same pointer!)
@@ -150,7 +158,8 @@ gchar** r_slot_get_root_classes(GHashTable *slots);
  *
  * @return TRUE if slot was found, FALSE if not
  */
-gboolean r_slot_list_contains(GList *slotlist, const RaucSlot *testslot);
+gboolean r_slot_list_contains(GList *slotlist, const RaucSlot *testslot)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Returns list of slots of given slot class
@@ -161,7 +170,8 @@ gboolean r_slot_list_contains(GList *slotlist, const RaucSlot *testslot);
  * @return list of pointers to all memers of slots hash table that are of
  *         selected class.
  */
-GList* r_slot_get_all_of_class(GHashTable *slots, const gchar* class);
+GList* r_slot_get_all_of_class(GHashTable *slots, const gchar* class)
+G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * Returns list of child slots of given parent slot.
@@ -172,4 +182,5 @@ GList* r_slot_get_all_of_class(GHashTable *slots, const gchar* class);
  * @return list of pointers to all memers of slots hash table that are childern
  *         of given slot.
  */
-GList* r_slot_get_all_children(GHashTable *slots, RaucSlot *parent);
+GList* r_slot_get_all_children(GHashTable *slots, RaucSlot *parent)
+G_GNUC_WARN_UNUSED_RESULT;
