@@ -127,8 +127,8 @@ gboolean determine_slot_states(GError **error)
 
 	for (GList *l = slotlist; l != NULL; l = l->next) {
 		g_autofree gchar *realdev = NULL;
-
-		RaucSlot *s = (RaucSlot*) g_hash_table_lookup(r_context()->config->slots, l->data);
+		RaucSlot *s = g_hash_table_lookup(r_context()->config->slots, l->data);
+		g_assert_nonnull(s);
 
 		if (g_strcmp0(s->bootname, r_context()->bootslot) == 0) {
 			booted = s;
@@ -152,25 +152,18 @@ gboolean determine_slot_states(GError **error)
 		}
 	}
 
-
-	if (booted) {
-		booted->state = ST_BOOTED;
-		g_debug("Found booted slot: %s on %s", booted->name, booted->device);
-	}
-
 	/* Determine active group members */
 	for (GList *l = slotlist; l != NULL; l = l->next) {
-		RaucSlot *s = (RaucSlot*) g_hash_table_lookup(r_context()->config->slots, l->data);
+		RaucSlot *s = g_hash_table_lookup(r_context()->config->slots, l->data);
+		g_assert_nonnull(s);
 
-		if (s->parent) {
-			if (s->parent->state & ST_ACTIVE) {
-				s->state |= ST_ACTIVE;
-			} else {
-				s->state |= ST_INACTIVE;
-			}
+		if (s == booted) {
+			s->state = ST_BOOTED;
+			g_debug("Found booted slot: %s on %s", s->name, s->device);
+		} else if (s->parent && s->parent == booted) {
+			s->state = ST_ACTIVE;
 		} else {
-			if (s->state == ST_UNKNOWN)
-				s->state |= ST_INACTIVE;
+			s->state = ST_INACTIVE;
 		}
 	}
 
@@ -222,9 +215,9 @@ gboolean determine_boot_states(GError **error)
 	return TRUE;
 }
 
-/* Returns newly allocated NULL-teminated string array of all classes listed in
+/* Returns NULL-teminated intern string array of all classes listed in
  * given manifest.
- * Free with g_strfreev */
+ * Free with g_free */
 static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
 {
 	GPtrArray *slotclasses = NULL;
@@ -289,7 +282,7 @@ static RaucSlot *select_inactive_slot_class_member(const gchar *rootclass)
  */
 GHashTable* determine_target_install_group(void)
 {
-	gchar **rootclasses = NULL;
+	g_autofree gchar **rootclasses = NULL;
 	GHashTable *targetgroup = NULL;
 	GHashTableIter iter;
 	RaucSlot *iterslot = NULL;
@@ -338,7 +331,7 @@ GHashTable* determine_target_install_group(void)
 GList* get_install_images(const RaucManifest *manifest, GHashTable *target_group, GError **error)
 {
 	GList *install_images = NULL;
-	gchar **slotclasses = NULL;
+	g_autofree gchar **slotclasses = NULL;
 
 	g_return_val_if_fail(manifest != NULL, NULL);
 	g_return_val_if_fail(target_group != NULL, NULL);
@@ -417,33 +410,40 @@ out:
 	return install_images;
 }
 
-static void parse_handler_output(gchar* line)
+static gchar* parse_handler_output(gchar* line)
 {
+	gchar *message = NULL;
 	g_auto(GStrv) split = NULL;
 
 	g_assert_nonnull(line);
 
 	if (!g_str_has_prefix(line, "<< ")) {
 		g_print("# %s\n", line);
-		return;
+		return NULL;
 	}
 
 	split = g_strsplit(line, " ", 5);
 
 	if (!split[1])
-		return;
+		return NULL;
 
 	if (g_strcmp0(split[1], "handler") == 0) {
-		g_print("Handler status: %s\n", split[2]);
+		message = g_strdup_printf("Handler status: %s", split[2]);
 	} else if (g_strcmp0(split[1], "image") == 0) {
-		g_print("Image '%s' status: %s\n", split[2], split[3]);
-	} else if (g_strcmp0(split[1], "error") == 0) {
-		g_print("error: '%s'\n", split[2]);
+		message = g_strdup_printf("Image '%s' status: %s", split[2], split[3]);
 	} else if (g_strcmp0(split[1], "bootloader") == 0) {
-		g_print("error: '%s'\n", split[2]);
+		message = g_strdup_printf("Bootloader status: %s", split[2]);
+	} else if (g_strcmp0(split[1], "error") == 0) {
+		g_autofree gchar *joined = g_strjoinv(" ", &split[2]);
+		message = g_strdup_printf("Error: %s", joined);
+	} else if (g_strcmp0(split[1], "debug") == 0) {
+		g_autofree gchar *joined = g_strjoinv(" ", &split[2]);
+		message = g_strdup_printf("Debug: %s", joined);
 	} else {
-		g_print("Unknown command: %s\n", split[1]);
+		message = g_strdup_printf("Unknown handler output: %s", line);
 	}
+
+	return message;
 }
 
 static gboolean verify_compatible(RaucInstallArgs *args, RaucManifest *manifest)
@@ -549,7 +549,7 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 	g_subprocess_launcher_setenv(launcher, "RAUC_TARGET_SLOTS", targetlist, TRUE);
 }
 
-static gboolean launch_and_wait_handler(gchar *update_source, gchar *handler_name, RaucManifest *manifest, GHashTable *target_group, GError **error)
+static gboolean launch_and_wait_handler(RaucInstallArgs *args, gchar *update_source, gchar *handler_name, RaucManifest *manifest, GHashTable *target_group, GError **error)
 {
 	g_autoptr(GSubprocessLauncher) handlelaunch = NULL;
 	g_autoptr(GSubprocess) handleproc = NULL;
@@ -558,15 +558,23 @@ static gboolean launch_and_wait_handler(gchar *update_source, gchar *handler_nam
 	GInputStream *instream = NULL;
 	g_autoptr(GDataInputStream) datainstream = NULL;
 	gchar *outline;
+	g_autoptr(GString) handler_args = NULL;
 
 	handlelaunch = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_STDOUT_PIPE | G_SUBPROCESS_FLAGS_STDERR_MERGE);
 
 	prepare_environment(handlelaunch, update_source, manifest, target_group);
 
+	handler_args = g_string_new(manifest->handler_args);
+	if (r_context()->handlerextra) {
+		if (handler_args->len)
+			g_string_append_c(handler_args, ' ');
+		g_string_append(handler_args, r_context()->handlerextra);
+	}
+
 	handleproc = g_subprocess_launcher_spawn(
 			handlelaunch, &ierror,
 			handler_name,
-			manifest->handler_args,
+			handler_args->str,
 			NULL);
 	if (handleproc == NULL) {
 		g_propagate_error(error, ierror);
@@ -577,11 +585,14 @@ static gboolean launch_and_wait_handler(gchar *update_source, gchar *handler_nam
 	datainstream = g_data_input_stream_new(instream);
 
 	do {
+		g_autofree gchar *handler_message = NULL;
 		outline = g_data_input_stream_read_line(datainstream, NULL, NULL, NULL);
 		if (!outline)
 			continue;
 
-		parse_handler_output(outline);
+		handler_message = parse_handler_output(outline);
+		if (handler_message != NULL)
+			install_args_update(args, handler_message);
 
 		g_free(outline);
 	} while (outline);
@@ -700,7 +711,7 @@ static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bun
 
 	handler_name = g_build_filename(bundledir, manifest->handler_name, NULL);
 
-	res = launch_and_wait_handler(bundledir, handler_name, manifest, target_group, error);
+	res = launch_and_wait_handler(args, bundledir, handler_name, manifest, target_group, error);
 
 out:
 	r_context_end_step("launch_and_wait_custom_handler", res);
@@ -842,15 +853,6 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 		mfimage = l->data;
 		dest_slot = g_hash_table_lookup(target_group, mfimage->slotclass);
 
-		/* Verify image checksum (for non-casync images) */
-		if (!g_str_has_suffix(mfimage->filename, ".caibx") && !g_str_has_suffix(mfimage->filename, ".caidx")) {
-			res = verify_checksum(&mfimage->checksum, mfimage->filename, &ierror);
-			if (!res) {
-				g_propagate_prefixed_error(error, ierror, "Failed verifying checksum: ");
-				goto out;
-			}
-		}
-
 		/* determine whether update image type is compatible with destination slot type */
 		update_handler = get_update_handler(mfimage, dest_slot, &ierror);
 		if (update_handler == NULL) {
@@ -1005,10 +1007,8 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	const gchar* bundlefile = args->name;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	g_autoptr(RaucManifest) manifest = NULL;
 	g_autoptr(RaucBundle) bundle = NULL;
 	GHashTable *target_group;
-	g_autofree gchar* manifestpath = NULL;
 
 	g_assert_nonnull(bundlefile);
 	g_assert_null(r_context()->install_info->mounted_bundle);
@@ -1043,16 +1043,6 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 
 	r_context()->install_info->mounted_bundle = bundle;
 
-	manifestpath = g_build_filename(bundle->mount_point, "manifest.raucm", NULL);
-	res = load_manifest_file(manifestpath, &manifest, &ierror);
-	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed loading manifest: ");
-		goto umount;
-	}
-
 	target_group = determine_target_install_group();
 	if (!target_group) {
 		g_set_error_literal(error, R_INSTALL_ERROR, R_INSTALL_ERROR_TARGET_GROUP, "Could not determine target group");
@@ -1062,20 +1052,19 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 
 	if (r_context()->config->preinstall_handler) {
 		g_message("Starting pre install handler: %s", r_context()->config->preinstall_handler);
-		res = launch_and_wait_handler(bundle->mount_point, r_context()->config->preinstall_handler, manifest, target_group, &ierror);
+		res = launch_and_wait_handler(args, bundle->mount_point, r_context()->config->preinstall_handler, bundle->manifest, target_group, &ierror);
 		if (!res) {
 			g_propagate_prefixed_error(error, ierror, "Pre-install handler error: ");
 			goto umount;
 		}
 	}
 
-
-	if (manifest->handler_name) {
-		g_message("Using custom handler: %s", manifest->handler_name);
-		res = launch_and_wait_custom_handler(args, bundle->mount_point, manifest, target_group, &ierror);
+	if (bundle->manifest->handler_name) {
+		g_message("Using custom handler: %s", bundle->manifest->handler_name);
+		res = launch_and_wait_custom_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
 	} else {
 		g_debug("Using default installation handler");
-		res = launch_and_wait_default_handler(args, bundle->mount_point, manifest, target_group, &ierror);
+		res = launch_and_wait_default_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
 	}
 
 	if (!res) {
@@ -1085,7 +1074,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 
 	if (r_context()->config->postinstall_handler) {
 		g_message("Starting post install handler: %s", r_context()->config->postinstall_handler);
-		res = launch_and_wait_handler(bundle->mount_point, r_context()->config->postinstall_handler, manifest, target_group, &ierror);
+		res = launch_and_wait_handler(args, bundle->mount_point, r_context()->config->postinstall_handler, bundle->manifest, target_group, &ierror);
 		if (!res) {
 			g_propagate_prefixed_error(error, ierror, "Post-install handler error: ");
 			goto umount;
