@@ -35,6 +35,7 @@ gchar *output_format = NULL;
 gchar *signing_keyring = NULL;
 gchar *mksquashfs_args = NULL;
 gchar *casync_args = NULL;
+gchar *handler_args = NULL;
 gboolean utf8_supported = FALSE;
 
 static gchar* make_progress_line(gint percentage)
@@ -72,7 +73,7 @@ static gboolean install_notify(gpointer data)
 	g_mutex_lock(&args->status_mutex);
 	while (!g_queue_is_empty(&args->status_messages)) {
 		gchar *msg = g_queue_pop_head(&args->status_messages);
-		g_print("%s", msg);
+		g_print("%s\n", msg);
 		g_free(msg);
 	}
 	r_exit_status = args->status_result;
@@ -107,7 +108,7 @@ static void on_installer_changed(GDBusProxy *proxy, GVariant *changed,
 
 	g_mutex_lock(&args->status_mutex);
 	if (g_variant_lookup(changed, "Operation", "&s", &message)) {
-		g_queue_push_tail(&args->status_messages, g_strdup_printf("%s\n", message));
+		g_queue_push_tail(&args->status_messages, g_strdup(message));
 	} else if (g_variant_lookup(changed, "Progress", "(i&si)", &percentage, &message, &depth)) {
 		if (install_progressbar && isatty(STDOUT_FILENO)) {
 			g_autofree gchar *progress = make_progress_line(percentage);
@@ -117,12 +118,12 @@ static void on_installer_changed(GDBusProxy *proxy, GVariant *changed,
 			 * - print 2 lines
 			 * - move to previous line
 			 */
-			g_queue_push_tail(&args->status_messages, g_strdup_printf("\r\033[J%3"G_GINT32_FORMAT "%% %s\n%s\033[F", percentage, message, progress));
+			g_queue_push_tail(&args->status_messages, g_strdup_printf("\r\033[F\033[J%3"G_GINT32_FORMAT "%% %s\n%s", percentage, message, progress));
 		} else {
-			g_queue_push_tail(&args->status_messages, g_strdup_printf("%3"G_GINT32_FORMAT "%% %s\n", percentage, message));
+			g_queue_push_tail(&args->status_messages, g_strdup_printf("%3"G_GINT32_FORMAT "%% %s", percentage, message));
 		}
 	} else if (g_variant_lookup(changed, "LastError", "&s", &message) && message[0] != '\0') {
-		g_queue_push_tail(&args->status_messages, g_strdup_printf("%sLastError: %s\n", isatty(STDOUT_FILENO) ? "\033[J" : "", message));
+		g_queue_push_tail(&args->status_messages, g_strdup_printf("%sLastError: %s", isatty(STDOUT_FILENO) ? "\033[J" : "", message));
 	}
 	g_mutex_unlock(&args->status_mutex);
 
@@ -189,6 +190,12 @@ static gchar *resolve_bundle_path(char *path)
 	return g_steal_pointer(&bundlelocation);
 }
 
+static void print_progress_callback(gint percentage,
+		const gchar *message,
+		gint nesting_depth)
+{
+	g_print("%3"G_GINT32_FORMAT "%% %s\n", percentage, message);
+}
 
 static gboolean install_start(int argc, char **argv)
 {
@@ -263,6 +270,7 @@ static gboolean install_start(int argc, char **argv)
 			goto out_loop;
 		}
 	} else {
+		r_context_register_progress_callback(print_progress_callback);
 		install_run(args);
 	}
 
@@ -289,7 +297,7 @@ out_loop:
 
 	if (installer)
 		g_signal_handlers_disconnect_by_data(installer, args);
-	g_clear_pointer(&installer, g_object_unref);
+	g_clear_object(&installer);
 	install_args_free(args);
 
 out:
@@ -354,13 +362,6 @@ static gboolean bundle_start(int argc, char **argv)
 	g_debug("input directory: %s", inpath);
 	g_debug("output bundle: %s", outpath);
 
-	if (!update_manifest(argv[2], FALSE, &ierror)) {
-		g_printerr("Failed to update manifest: %s\n", ierror->message);
-		g_clear_error(&ierror);
-		r_exit_status = 1;
-		goto out;
-	}
-
 	if (!create_bundle(argv[3], argv[2], &ierror)) {
 		g_printerr("Failed to create bundle: %s\n", ierror->message);
 		g_clear_error(&ierror);
@@ -376,7 +377,7 @@ static gboolean write_slot_start(int argc, char **argv)
 {
 	GError *ierror = NULL;
 	g_autoptr(RaucImage) image = g_new0(RaucImage, 1);
-	g_autoptr(RaucSlot) slot = g_new0(RaucSlot, 1);
+	RaucSlot *slot = NULL;
 	g_autoptr(GFileInfo) info = NULL;
 	g_autoptr(GInputStream) instream = NULL;
 	g_autoptr(GFile) imagefile = NULL;
@@ -717,6 +718,13 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 	g_string_append_printf(text, "Hooks:      \t'%s'\n", hookstring);
 	g_free(hookstring);
 
+	g_string_append_printf(text, "Bundle Format: \t%s\n", r_manifest_bundle_format_to_str(manifest->bundle_format));
+	if (manifest->bundle_format == R_MANIFEST_FORMAT_VERITY) {
+		g_string_append_printf(text, "  Verity Salt: \t'%s'\n", manifest->bundle_verity_salt);
+		g_string_append_printf(text, "  Verity Hash: \t'%s'\n", manifest->bundle_verity_hash);
+		g_string_append_printf(text, "  Verity Size: \t%"G_GUINT64_FORMAT "\n", manifest->bundle_verity_size);
+	}
+
 	g_ptr_array_unref(hooks);
 
 	cnt = g_list_length(manifest->images);
@@ -858,10 +866,8 @@ static gchar* info_formatter_json_pretty(RaucManifest *manifest)
 static gboolean info_start(int argc, char **argv)
 {
 	g_autofree gchar* tmpdir = NULL;
-	g_autofree gchar* bundledir = NULL;
-	g_autofree gchar* manifestpath = NULL;
 	g_autofree gchar *bundlelocation = NULL;
-	RaucManifest *manifest = NULL;
+	g_autoptr(RaucManifest) manifest = NULL;
 	g_autoptr(RaucBundle) bundle = NULL;
 	GError *error = NULL;
 	gboolean res = FALSE;
@@ -900,9 +906,6 @@ static gboolean info_start(int argc, char **argv)
 		goto out;
 	}
 
-	bundledir = g_build_filename(tmpdir, "bundle-content", NULL);
-	manifestpath = g_build_filename(bundledir, "manifest.raucm", NULL);
-
 	bundlelocation = resolve_bundle_path(argv[2]);
 	if (bundlelocation == NULL)
 		goto out;
@@ -915,18 +918,15 @@ static gboolean info_start(int argc, char **argv)
 		goto out;
 	}
 
-	res = extract_file_from_bundle(bundle, bundledir, "manifest.raucm", &error);
-	if (!res) {
-		g_printerr("%s\n", error->message);
-		g_clear_error(&error);
-		goto out;
-	}
-
-	res = load_manifest_file(manifestpath, &manifest, &error);
-	if (!res) {
-		g_printerr("%s\n", error->message);
-		g_clear_error(&error);
-		goto out;
+	if (bundle->manifest) {
+		manifest = g_steal_pointer(&bundle->manifest);
+	} else {
+		res = load_manifest_from_bundle(bundle, &manifest, &error);
+		if (!res) {
+			g_printerr("%s\n", error->message);
+			g_clear_error(&error);
+			goto out;
+		}
 	}
 
 	text = formatter(manifest);
@@ -945,7 +945,7 @@ static gboolean info_start(int argc, char **argv)
 			goto out;
 		}
 
-		text = print_cert_chain(bundle->verified_chain);
+		text = format_cert_chain(bundle->verified_chain);
 		g_print("%s\n", text);
 		g_free(text);
 	}
@@ -1059,7 +1059,7 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 {
 	GString *text = g_string_new(NULL);
 	RaucSlot *bootedfrom = NULL;
-	gchar **slotclasses = NULL;
+	g_autofree gchar **slotclasses = NULL;
 
 	g_return_val_if_fail(status, NULL);
 
@@ -1657,7 +1657,7 @@ static gboolean status_start(int argc, char **argv)
 out:
 	if (message)
 		g_message("rauc status: %s", message);
-	g_clear_pointer(&proxy, g_object_unref);
+	g_clear_object(&proxy);
 
 	return TRUE;
 }
@@ -1704,7 +1704,11 @@ typedef struct {
 
 static GOptionEntry entries_install[] = {
 	{"ignore-compatible", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_compatible, "disable compatible check", NULL},
+#if ENABLE_SERVICE == 1
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
+#else
+	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+#endif
 	{0}
 };
 
@@ -1740,12 +1744,18 @@ static GOptionEntry entries_status[] = {
 	{0}
 };
 
+static GOptionEntry entries_service[] = {
+	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+	{0}
+};
+
 static GOptionGroup *install_group;
 static GOptionGroup *bundle_group;
 static GOptionGroup *resign_group;
 static GOptionGroup *convert_group;
 static GOptionGroup *info_group;
 static GOptionGroup *status_group;
+static GOptionGroup *service_group;
 
 static void create_option_groups(void)
 {
@@ -1768,13 +1778,16 @@ static void create_option_groups(void)
 
 	status_group  = g_option_group_new("status", "Status options:", "help dummy", NULL, NULL);
 	g_option_group_add_entries(status_group, entries_status);
+
+	service_group  = g_option_group_new("service", "Service options:", "help dummy", NULL, NULL);
+	g_option_group_add_entries(service_group, entries_service);
 }
 
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
 	gchar *confpath = NULL, *certpath = NULL, *keypath = NULL, *keyring = NULL, **intermediate = NULL, *mount = NULL,
-	      *handlerextra = NULL, *bootslot = NULL;
+	      *bootslot = NULL;
 	char *cmdarg = NULL;
 	g_autoptr(GOptionContext) context = NULL;
 	GOptionEntry entries[] = {
@@ -1785,7 +1798,6 @@ static void cmdline_handler(int argc, char **argv)
 		{"intermediate", '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &intermediate, "intermediate CA file name", "PEMFILE"},
 		{"mount", '\0', 0, G_OPTION_ARG_FILENAME, &mount, "mount prefix", "PATH"},
 		{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
-		{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handlerextra, "extra handler arguments", "ARGS"},
 		{"debug", 'd', 0, G_OPTION_ARG_NONE, &debug, "enable debug output", NULL},
 		{"version", '\0', 0, G_OPTION_ARG_NONE, &version, "display version", NULL},
 		{"help", 'h', 0, G_OPTION_ARG_NONE, &help, NULL, NULL},
@@ -1800,7 +1812,7 @@ static void cmdline_handler(int argc, char **argv)
 		{INSTALL, "install", "install <BUNDLE>", "Install a bundle", install_start, install_group, FALSE},
 #if ENABLE_CREATE == 1
 		{BUNDLE, "bundle", "bundle <INPUTDIR> <BUNDLENAME>", "Create a bundle from a content directory", bundle_start, bundle_group, FALSE},
-		{RESIGN, "resign", "resign <BUNDLENAME>", "Resign an already signed bundle", resign_start, resign_group, FALSE},
+		{RESIGN, "resign", "resign <INBUNDLE> <OUTBUNDLE>", "Resign an already signed bundle", resign_start, resign_group, FALSE},
 		{CONVERT, "convert", "convert <INBUNDLE> <OUTBUNDLE>", "Convert to casync index bundle and store", convert_start, convert_group, FALSE},
 #endif
 		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>", "Extract the bundle content", extract_start, NULL, FALSE},
@@ -1814,7 +1826,7 @@ static void cmdline_handler(int argc, char **argv)
 		 status_start, status_group, TRUE},
 		{WRITE_SLOT, "write-slot", "write-slot <SLOTNAME> <IMAGE>", "Write image to slot and bypass all update logic", write_slot_start, NULL, FALSE},
 #if ENABLE_SERVICE == 1
-		{SERVICE, "service", "service", "Start RAUC service", service_start, NULL, TRUE},
+		{SERVICE, "service", "service", "Start RAUC service", service_start, service_group, TRUE},
 #endif
 		{0}
 	};
@@ -1951,8 +1963,8 @@ static void cmdline_handler(int argc, char **argv)
 			r_context_conf()->mountprefix = mount;
 		if (bootslot)
 			r_context_conf()->bootslot = bootslot;
-		if (handlerextra)
-			r_context_conf()->handlerextra = handlerextra;
+		if (handler_args)
+			r_context_conf()->handlerextra = handler_args;
 	} else {
 		if (confpath != NULL ||
 		    certpath != NULL ||
@@ -1982,6 +1994,12 @@ print_help:
 
 int main(int argc, char **argv)
 {
+	GLogLevelFlags fatal_mask;
+
+	fatal_mask = g_log_set_always_fatal(G_LOG_FATAL_MASK);
+	fatal_mask |= G_LOG_LEVEL_CRITICAL;
+	g_log_set_always_fatal(fatal_mask);
+
 	/* disable remote VFS */
 	g_assert(g_setenv("GIO_USE_VFS", "local", TRUE));
 
