@@ -99,6 +99,7 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	GError *ierror = NULL;
 	RaucManifest *raucm = g_new0(RaucManifest, 1);
 	gboolean res = FALSE;
+	g_autofree gchar *tmp = NULL;
 	gchar **groups;
 	gsize group_count;
 	gchar **bundle_hooks;
@@ -121,6 +122,29 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	}
 	g_key_file_remove_group(key_file, "update", NULL);
 
+	/* parse [bundle] section */
+	tmp = key_file_consume_string(key_file, "bundle", "format", NULL);
+	if (tmp == NULL || g_strcmp0(tmp, "plain") == 0) {
+		raucm->bundle_format = R_MANIFEST_FORMAT_PLAIN;
+	} else if (g_strcmp0(tmp, "verity") == 0) {
+		/* only SHA256 is supported for now */
+		raucm->bundle_format = R_MANIFEST_FORMAT_VERITY;
+		raucm->bundle_verity_hash = key_file_consume_string(key_file, "bundle", "verity-hash", NULL);
+		raucm->bundle_verity_salt = key_file_consume_string(key_file, "bundle", "verity-salt", NULL);
+		raucm->bundle_verity_size = g_key_file_get_uint64(key_file, "bundle", "verity-size", NULL);
+		/* values are checked in check_manifest */
+		g_key_file_remove_key(key_file, "bundle", "verity-size", NULL);
+	} else {
+		g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+				"Invalid format value '%s' in group '[bundle]'", tmp);
+		goto free;
+	}
+	if (!check_remaining_keys(key_file, "bundle", &ierror)) {
+		g_propagate_error(error, ierror);
+		goto free;
+	}
+	g_key_file_remove_group(key_file, "bundle", NULL);
+
 	/* parse [keyring] section */
 	raucm->keyring = key_file_consume_string(key_file, "keyring", "archive", NULL);
 	if (!check_remaining_keys(key_file, "keyring", &ierror)) {
@@ -132,14 +156,6 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	/* parse [handler] section */
 	raucm->handler_name = key_file_consume_string(key_file, "handler", "filename", NULL);
 	raucm->handler_args = key_file_consume_string(key_file, "handler", "args", NULL);
-	if (r_context()->handlerextra) {
-		GString *str = g_string_new(raucm->handler_args);
-		if (str->len)
-			g_string_append_c(str, ' ');
-		g_string_append(str, r_context()->handlerextra);
-		g_free(raucm->handler_args);
-		raucm->handler_args = g_string_free(str, FALSE);
-	}
 	if (!check_remaining_keys(key_file, "handler", &ierror)) {
 		g_propagate_error(error, ierror);
 		goto free;
@@ -276,8 +292,6 @@ gboolean load_manifest_file(const gchar *filename, RaucManifest **manifest, GErr
 	g_autoptr(GKeyFile) key_file = NULL;
 	gboolean res = FALSE;
 
-	r_context_begin_step("load_manifest_file", "Loading manifest file", 0);
-
 	key_file = g_key_file_new();
 
 	res = g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror);
@@ -293,14 +307,163 @@ gboolean load_manifest_file(const gchar *filename, RaucManifest **manifest, GErr
 	}
 
 out:
-	r_context_end_step("load_manifest_file", res);
 	return res;
 }
 
-gboolean save_manifest_file(const gchar *filename, RaucManifest *mf, GError **error)
+static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
+{
+	gboolean res = FALSE;
+
+	switch (mf->bundle_format) {
+		case R_MANIFEST_FORMAT_PLAIN:
+			break; /* no additional data needed */
+		case R_MANIFEST_FORMAT_VERITY:
+			break; /* data checked in _detached/_inline */
+		default: {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format");
+			goto out;
+		}
+	}
+
+	for (GList *l = mf->images; l != NULL; l = l->next) {
+		RaucImage *image = l->data;
+
+		g_assert(image);
+		g_assert(image->filename);
+
+		if (image->checksum.type != G_CHECKSUM_SHA256) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported checksum algorithm for image %s", image->filename);
+			goto out;
+		}
+		if (!image->checksum.digest) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing digest for image %s", image->filename);
+			goto out;
+		}
+		if (!image->checksum.size) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for image %s", image->filename);
+			goto out;
+		}
+	}
+
+	res = TRUE;
+out:
+	return res;
+}
+
+gboolean check_manifest_internal(const RaucManifest *mf, GError **error)
+{
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+
+	r_context_begin_step("check_manifest", "Checking manifest contents", 0);
+
+	res = check_manifest_common(mf, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	switch (mf->bundle_format) {
+		case R_MANIFEST_FORMAT_PLAIN:
+			break; /* no additional data needed */
+		case R_MANIFEST_FORMAT_VERITY: {
+			if (mf->bundle_verity_hash) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for verity bundle in internal manifest");
+				goto out;
+			}
+			if (mf->bundle_verity_salt) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for verity bundle in internal manifest");
+				goto out;
+			}
+			if (mf->bundle_verity_size) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected hash for verity bundle in internal manifest");
+				goto out;
+			}
+
+			break;
+		};
+		default: {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format");
+			goto out;
+		}
+	}
+
+	res = TRUE;
+out:
+	r_context_end_step("check_manifest", res);
+	return res;
+}
+
+gboolean check_manifest_external(const RaucManifest *mf, GError **error)
+{
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+
+	r_context_begin_step("check_manifest", "Checking manifest contents", 0);
+
+	res = check_manifest_common(mf, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	switch (mf->bundle_format) {
+		case R_MANIFEST_FORMAT_PLAIN: {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format 'plain' for external manifest");
+			goto out;
+		}
+		case R_MANIFEST_FORMAT_VERITY: {
+			guint8 *tmp;
+
+			if (!mf->bundle_verity_hash) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing hash for verity bundle");
+				goto out;
+			}
+			tmp = r_hex_decode(mf->bundle_verity_hash, 32);
+			if (!tmp) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Invalid hash for verity bundle");
+				goto out;
+			}
+			g_free(tmp);
+
+			if (!mf->bundle_verity_salt) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing salt for verity bundle");
+				goto out;
+			}
+			tmp = r_hex_decode(mf->bundle_verity_salt, 32);
+			if (!tmp) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Invalid salt for verity bundle");
+				goto out;
+			}
+			g_free(tmp);
+
+			if (!mf->bundle_verity_size) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for verity bundle");
+				goto out;
+			}
+
+			if (mf->bundle_verity_size % 4096) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unaligned size for verity bundle");
+				goto out;
+			}
+
+			break;
+		};
+		default: {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format");
+			goto out;
+		}
+	}
+
+	res = TRUE;
+out:
+	r_context_end_step("check_manifest", res);
+	return res;
+}
+
+static GKeyFile *prepare_manifest(const RaucManifest *mf)
 {
 	g_autoptr(GKeyFile) key_file = NULL;
-	gboolean res = FALSE;
 	GPtrArray *hooks = g_ptr_array_new_full(3, g_free);
 
 	key_file = g_key_file_new();
@@ -317,6 +480,24 @@ gboolean save_manifest_file(const gchar *filename, RaucManifest *mf, GError **er
 	if (mf->update_build)
 		g_key_file_set_string(key_file, "update", "build", mf->update_build);
 
+	switch (mf->bundle_format) {
+		case R_MANIFEST_FORMAT_PLAIN:
+			break;
+		case R_MANIFEST_FORMAT_VERITY: {
+			g_key_file_set_string(key_file, "bundle", "format", "verity");
+			if (mf->bundle_verity_hash)
+				g_key_file_set_string(key_file, "bundle", "verity-hash", mf->bundle_verity_hash);
+			if (mf->bundle_verity_salt)
+				g_key_file_set_string(key_file, "bundle", "verity-salt", mf->bundle_verity_salt);
+			if (mf->bundle_verity_size)
+				g_key_file_set_uint64(key_file, "bundle", "verity-size", mf->bundle_verity_size);
+
+			break;
+		};
+		default:
+			break;
+	}
+
 	if (mf->keyring)
 		g_key_file_set_string(key_file, "keyring", "archive", mf->keyring);
 
@@ -337,7 +518,6 @@ gboolean save_manifest_file(const gchar *filename, RaucManifest *mf, GError **er
 		g_key_file_set_string_list(key_file, "hooks", "hooks",
 				(const gchar **)hooks->pdata, hooks->len);
 	}
-
 	g_ptr_array_unref(hooks);
 
 	for (GList *l = mf->images; l != NULL; l = l->next) {
@@ -399,11 +579,42 @@ gboolean save_manifest_file(const gchar *filename, RaucManifest *mf, GError **er
 			g_key_file_set_string(key_file, group, "filename", file->filename);
 	}
 
-	res = g_key_file_save_to_file(key_file, filename, NULL);
+	return g_steal_pointer(&key_file);
+}
+
+gboolean save_manifest_mem(GBytes **mem, const RaucManifest *mf)
+{
+	g_autoptr(GKeyFile) key_file = NULL;
+	guint8 *data = NULL;
+	gsize length = 0;
+
+	g_return_val_if_fail(mem != NULL && *mem == NULL, FALSE);
+	g_return_val_if_fail(mf != NULL, FALSE);
+
+	key_file = prepare_manifest(mf);
+
+	/* according to the docs, this never fails */
+	data = (guint8*)g_key_file_to_data(key_file, &length, NULL);
+	g_assert(data != NULL);
+	g_assert(length > 0);
+
+	*mem = g_bytes_new(data, length);
+
+	return TRUE;
+}
+
+gboolean save_manifest_file(const gchar *filename, const RaucManifest *mf, GError **error)
+{
+	GError *ierror = NULL;
+	g_autoptr(GKeyFile) key_file = NULL;
+	gboolean res = FALSE;
+
+	key_file = prepare_manifest(mf);
+
+	res = g_key_file_save_to_file(key_file, filename, &ierror);
 	if (!res)
-		goto free;
+		g_propagate_error(error, ierror);
 
-free:
 	return res;
 }
 
@@ -414,6 +625,7 @@ void r_free_image(gpointer data)
 	g_return_if_fail(image);
 
 	g_free(image->slotclass);
+	g_free(image->variant);
 	g_free(image->checksum.digest);
 	g_free(image->filename);
 	g_free(image);
@@ -440,6 +652,8 @@ void free_manifest(RaucManifest *manifest)
 	g_free(manifest->update_version);
 	g_free(manifest->update_description);
 	g_free(manifest->update_build);
+	g_free(manifest->bundle_verity_hash);
+	g_free(manifest->bundle_verity_salt);
 	g_free(manifest->keyring);
 	g_free(manifest->handler_name);
 	g_free(manifest->handler_args);
@@ -449,7 +663,7 @@ void free_manifest(RaucManifest *manifest)
 	g_free(manifest);
 }
 
-static gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GError **error)
+gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = TRUE;
@@ -486,57 +700,3 @@ static gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *d
 
 	return res;
 }
-
-gboolean update_manifest(const gchar *dir, gboolean signature, GError **error)
-{
-	GError *ierror = NULL;
-	g_autofree gchar* manifestpath = g_build_filename(dir, "manifest.raucm", NULL);
-	g_autofree gchar* signaturepath = g_build_filename(dir, "manifest.raucm.sig", NULL);
-	g_autoptr(RaucManifest) manifest = NULL;
-	g_autoptr(GBytes) sig = NULL;
-	gboolean res = FALSE;
-
-	if (signature) {
-		g_assert_nonnull(r_context()->certpath);
-		g_assert_nonnull(r_context()->keypath);
-	}
-
-	res = load_manifest_file(manifestpath, &manifest, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
-	}
-
-	res = update_manifest_checksums(manifest, dir, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
-	}
-
-	res = save_manifest_file(manifestpath, manifest, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
-	}
-
-	if (signature) {
-		sig = cms_sign_file(manifestpath,
-				r_context()->certpath,
-				r_context()->keypath,
-				NULL,
-				&ierror);
-		if (sig == NULL) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-
-		res = write_file(signaturepath, sig, &ierror);
-		if (!res) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-	}
-
-out:
-	return res;
-}
