@@ -7,6 +7,7 @@
 #include <openssl/crypto.h>
 #include <openssl/engine.h>
 #include <openssl/x509.h>
+#include <string.h>
 
 #include "context.h"
 #include "signature.h"
@@ -453,7 +454,7 @@ X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **e
 	return g_steal_pointer(&store);
 }
 
-GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
+GBytes *cms_sign(GBytes *content, gboolean detached, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
 {
 	GError *ierror = NULL;
 	BIO *incontent = BIO_new_mem_buf((void *)g_bytes_get_data(content, NULL),
@@ -464,7 +465,7 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 	STACK_OF(X509) *intercerts = NULL;
 	CMS_ContentInfo *cms = NULL;
 	GBytes *res = NULL;
-	int flags = CMS_DETACHED | CMS_BINARY;
+	int flags = CMS_BINARY | CMS_NOSMIMECAP;
 	const gchar *keyring_path = NULL, *keyring_dir = NULL;
 
 	g_return_val_if_fail(content != NULL, NULL);
@@ -496,6 +497,9 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 		sk_X509_push(intercerts, intercert);
 	}
 
+	if (detached)
+		flags |= CMS_DETACHED;
+
 	cms = CMS_sign(signcert, pkey, intercerts, incontent, flags);
 	if (cms == NULL) {
 		unsigned long err;
@@ -542,6 +546,7 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 		g_autoptr(CMS_ContentInfo) vcms = NULL;
 		g_autoptr(X509_STORE) store = NULL;
 		STACK_OF(X509) *verified_chain = NULL;
+		g_autoptr(GBytes) manifest = NULL;
 
 		if (!(store = setup_x509_store(keyring_path, keyring_dir, &ierror))) {
 			g_propagate_error(error, ierror);
@@ -550,12 +555,19 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 		}
 
 		g_message("Keyring given, doing signature verification");
-		if (!cms_verify(content, res, store, &vcms, &ierror)) {
-			g_propagate_error(error, ierror);
-			g_clear_pointer(&res, g_bytes_unref);
-			goto out;
+		if (detached) {
+			if (!cms_verify_bytes(content, res, store, &vcms, NULL, &ierror)) {
+				g_propagate_error(error, ierror);
+				g_clear_pointer(&res, g_bytes_unref);
+				goto out;
+			}
+		} else {
+			if (!cms_verify_bytes(NULL, res, store, &vcms, &manifest, &ierror)) {
+				g_propagate_error(error, ierror);
+				g_clear_pointer(&res, g_bytes_unref);
+				goto out;
+			}
 		}
-
 		if (!cms_get_cert_chain(vcms, store, &verified_chain, &ierror)) {
 			g_propagate_error(error, ierror);
 			g_clear_pointer(&res, g_bytes_unref);
@@ -732,27 +744,88 @@ static gchar* get_cert_time(const ASN1_TIME *time)
 	return ret;
 }
 
-gchar* print_cert_chain(STACK_OF(X509) *verified_chain)
+static gchar* bio_mem_unwrap(BIO *mem)
 {
-	GString *text = NULL;
-	char buf[BUFSIZ];
+	long size;
+	gchar *data, *ret;
+
+	g_return_val_if_fail(mem != NULL, NULL);
+
+	size = BIO_get_mem_data(mem, &data);
+	ret = g_strndup(data, size);
+	BIO_free(mem);
+
+	return ret;
+}
+
+gchar* format_cert_chain(STACK_OF(X509) *verified_chain)
+{
+	BIO *text = NULL;
+	gchar *tmp = NULL;
 
 	g_return_val_if_fail(verified_chain != NULL, NULL);
 
-	text = g_string_new("Certificate Chain:\n");
+	text = BIO_new(BIO_s_mem());
+	BIO_printf(text, "Certificate Chain:\n");
 	for (int i = 0; i < sk_X509_num(verified_chain); i++) {
-		X509_NAME_oneline(X509_get_subject_name(sk_X509_value(verified_chain, i)),
-				buf, sizeof buf);
-		g_string_append_printf(text, "%2d Subject: %s\n", i, buf);
-		X509_NAME_oneline(X509_get_issuer_name(sk_X509_value(verified_chain, i)),
-				buf, sizeof buf);
-		g_string_append_printf(text, "   Issuer: %s\n", buf);
-		g_string_append_printf(text, "   SPKI sha256: %s\n", get_pubkey_hash(sk_X509_value(verified_chain, i)));
-		g_string_append_printf(text, "   Not Before: %s\n", get_cert_time(X509_get0_notBefore((const X509*) sk_X509_value(verified_chain, i))));
-		g_string_append_printf(text, "   Not After:  %s\n", get_cert_time(X509_get0_notAfter((const X509*) sk_X509_value(verified_chain, i))));
+		BIO_printf(text, "%2d Subject: ", i);
+		X509_NAME_print_ex(text, X509_get_subject_name(sk_X509_value(verified_chain, i)), 0, XN_FLAG_ONELINE);
+		BIO_printf(text, "\n");
+
+		BIO_printf(text, "   Issuer: ");
+		X509_NAME_print_ex(text, X509_get_issuer_name(sk_X509_value(verified_chain, i)), 0, XN_FLAG_ONELINE);
+		BIO_printf(text, "\n");
+
+		tmp = get_pubkey_hash(sk_X509_value(verified_chain, i));
+		BIO_printf(text, "   SPKI sha256: %s\n", tmp);
+		g_free(tmp);
+
+		tmp = get_cert_time(X509_get0_notBefore((const X509*) sk_X509_value(verified_chain, i)));
+		BIO_printf(text, "   Not Before: %s\n", tmp);
+		g_free(tmp);
+
+		tmp = get_cert_time(X509_get0_notAfter((const X509*) sk_X509_value(verified_chain, i)));
+		BIO_printf(text, "   Not After:  %s\n", tmp);
+		g_free(tmp);
 	}
 
-	return g_string_free(text, FALSE);
+	return bio_mem_unwrap(text);
+}
+
+static gchar *cms_get_signers(CMS_ContentInfo *cms, GError **error)
+{
+	STACK_OF(X509) *signers = NULL;
+	BIO *text = NULL;
+
+	g_return_val_if_fail(cms != NULL, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	signers = CMS_get0_signers(cms);
+	if (signers == NULL) {
+		g_set_error_literal(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_GET_SIGNER,
+				"Failed to obtain signer info");
+		goto out;
+	}
+
+	text = BIO_new(BIO_s_mem());
+	BIO_printf(text, "'");
+	for (int i = 0; i < sk_X509_num(signers); i++) {
+		if (i)
+			BIO_printf(text, "', '");
+		X509_NAME_print_ex(text, X509_get_subject_name(sk_X509_value(signers, i)), 0, XN_FLAG_ONELINE);
+	}
+	BIO_printf(text, "'");
+
+out:
+	if (signers)
+		sk_X509_free(signers);
+	if (text)
+		return bio_mem_unwrap(text);
+	else
+		return NULL;
 }
 
 gboolean cms_get_cert_chain(CMS_ContentInfo *cms, X509_STORE *store, STACK_OF(X509) **verified_chain, GError **error)
@@ -867,19 +940,135 @@ static gboolean asn1_time_to_tm(const ASN1_TIME *intime, struct tm *tm)
 	return TRUE;
 }
 
-gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
+static void debug_cms_ci(CMS_ContentInfo *cms)
 {
-	CMS_ContentInfo *icms = NULL;
-	BIO *incontent = BIO_new_mem_buf((void *)g_bytes_get_data(content, NULL),
-			g_bytes_get_size(content));
+	BIO *out = BIO_new(BIO_s_mem());
+	gchar *out_str = NULL;
+	long size;
+
+	CMS_ContentInfo_print_ctx(out, cms, 2, NULL);
+	if ((size = BIO_get_mem_data(out, &out_str)) > 0) {
+		/* replace final newline with nul */
+		out_str[size-1] = '\0';
+		g_log(G_LOG_DOMAIN "-signature", G_LOG_LEVEL_DEBUG, "\n%s", out_str);
+	}
+	BIO_free_all(out);
+}
+
+gboolean cms_is_detached(GBytes *sig, gboolean *detached, GError **error)
+{
+	CMS_ContentInfo *cms = NULL;
 	BIO *insig = BIO_new_mem_buf((void *)g_bytes_get_data(sig, NULL),
 			g_bytes_get_size(sig));
 	gboolean res = FALSE;
 
-	g_return_val_if_fail(content != NULL, FALSE);
+	g_return_val_if_fail(sig != NULL, FALSE);
+	g_return_val_if_fail(detached != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (!(cms = d2i_CMS_bio(insig, NULL))) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"failed to parse signature");
+		goto out;
+	}
+
+	*detached = CMS_is_detached(cms);
+
+	res = TRUE;
+
+out:
+	if (cms)
+		CMS_ContentInfo_free(cms);
+	return res;
+}
+
+gboolean cms_get_unverified_manifest(GBytes *sig, GBytes **manifest, GError **error)
+{
+	CMS_ContentInfo *cms = NULL;
+	BIO *insig = BIO_new_mem_buf((void *)g_bytes_get_data(sig, NULL),
+			g_bytes_get_size(sig));
+	ASN1_OCTET_STRING **content = NULL;
+	GBytes *tmp = NULL;
+	gboolean res = FALSE;
+
+	g_return_val_if_fail(sig != NULL, FALSE);
+	g_return_val_if_fail(manifest != NULL && *manifest == NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (!(cms = d2i_CMS_bio(insig, NULL))) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"failed to parse signature");
+		goto out;
+	}
+
+	content = CMS_get0_content(cms);
+	if (!content) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"unsupported signature content type");
+		goto out;
+	}
+	if (!(*content)) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"missing manifest in inline signature");
+		goto out;
+	}
+	if (!(*content)->data || ((*content)->length <= 0)) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"invalid manifest length in inline signature");
+		goto out;
+	}
+
+	tmp = g_bytes_new((*content)->data, (*content)->length);
+	if (!tmp) {
+		g_set_error_literal(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_UNKNOWN,
+				"failed to get manifest from inline signature");
+		goto out;
+	}
+	*manifest = tmp;
+
+	res = TRUE;
+
+out:
+	if (cms)
+		CMS_ContentInfo_free(cms);
+	return res;
+}
+
+gboolean cms_verify_bytes(GBytes *content, GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GBytes **manifest, GError **error)
+{
+	GError *ierror = NULL;
+	CMS_ContentInfo *icms = NULL;
+	BIO *incontent = NULL;
+	BIO *insig = BIO_new_mem_buf((void *)g_bytes_get_data(sig, NULL),
+			g_bytes_get_size(sig));
+	BIO *outcontent = BIO_new(BIO_s_mem());
+	g_autofree gchar *signers = NULL;
+	gboolean res = FALSE;
+	gboolean verified = FALSE;
+	gboolean detached;
+
 	g_return_val_if_fail(sig != NULL, FALSE);
 	g_return_val_if_fail(store != NULL, FALSE);
 	g_return_val_if_fail(cms == NULL || *cms == NULL, FALSE);
+	g_return_val_if_fail(manifest == NULL || *manifest == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	r_context_begin_step("cms_verify", "Verifying signature", 0);
@@ -893,6 +1082,51 @@ gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_Content
 		goto out;
 	}
 
+	debug_cms_ci(icms);
+
+	detached = CMS_is_detached(icms);
+	if (detached) {
+		if (content == NULL) {
+			/* we have a detached signature but no content to verify */
+			g_set_error(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_INVALID,
+					"no content provided for detached signature");
+			goto out;
+		}
+		if (manifest != NULL) {
+			/* we have a detached signature but a place for the manifest */
+			g_set_error(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_INVALID,
+					"unexpected manifest output location for detached signature");
+			goto out;
+		}
+		incontent = BIO_new_mem_buf((void *)g_bytes_get_data(content, NULL),
+				g_bytes_get_size(content));
+	} else {
+		if (content != NULL) {
+			/* we have an inline signature but some content to verify */
+			g_set_error(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_INVALID,
+					"unexpected content provided for inline signature");
+			goto out;
+		}
+		if (manifest == NULL) {
+			/* we have an inline signature but no place to store manifest */
+			g_set_error(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_INVALID,
+					"no manifest output location for inline signature");
+			goto out;
+		}
+	}
+
 	/* Optionally use certificate signing timestamp for verification */
 	if (r_context()->config->use_bundle_signing_time) {
 		STACK_OF(CMS_SignerInfo) *sinfos;
@@ -926,7 +1160,11 @@ gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_Content
 		X509_VERIFY_PARAM_free(param);
 	}
 
-	if (!CMS_verify(icms, NULL, store, incontent, NULL, CMS_DETACHED | CMS_BINARY)) {
+	if (detached)
+		verified = CMS_verify(icms, NULL, store, incontent, NULL, CMS_DETACHED | CMS_BINARY);
+	else
+		verified = CMS_verify(icms, NULL, store, NULL, outcontent, CMS_BINARY);
+	if (!verified) {
 		unsigned long err;
 		const gchar *data;
 		int flags;
@@ -939,6 +1177,26 @@ gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_Content
 		goto out;
 	}
 
+	signers = cms_get_signers(icms, &ierror);
+	if (!signers) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+	g_message("Verified %s signature by %s", detached ? "detached" : "inline", signers);
+
+	if (!detached) {
+		GBytes *tmp = bytes_from_bio(outcontent);
+		if (!tmp) {
+			g_set_error_literal(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_UNKNOWN,
+					"missing manifest in inline signature");
+			goto out;
+		}
+		*manifest = tmp;
+	}
+
 	if (cms)
 		*cms = icms;
 
@@ -947,6 +1205,7 @@ out:
 	ERR_print_errors_fp(stdout);
 	BIO_free_all(incontent);
 	BIO_free_all(insig);
+	BIO_free_all(outcontent);
 	if (!cms)
 		CMS_ContentInfo_free(icms);
 	r_context_end_step("cms_verify", res);
@@ -972,7 +1231,7 @@ GBytes *cms_sign_file(const gchar *filename, const gchar *certfile, const gchar
 	}
 	content = g_mapped_file_get_bytes(file);
 
-	sig = cms_sign(content, certfile, keyfile, interfiles, &ierror);
+	sig = cms_sign(content, TRUE, certfile, keyfile, interfiles, &ierror);
 	if (sig == NULL) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -982,20 +1241,51 @@ out:
 	return sig;
 }
 
-gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
+GBytes *cms_sign_manifest(RaucManifest *manifest, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
+{
+	GError *ierror = NULL;
+	g_autoptr(GMappedFile) file = NULL;
+	g_autoptr(GBytes) content = NULL;
+	GBytes *sig = NULL;
+
+	g_return_val_if_fail(manifest != NULL, FALSE);
+	g_return_val_if_fail(certfile != NULL, FALSE);
+	g_return_val_if_fail(keyfile != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (!save_manifest_mem(&content, manifest)) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_UNKNOWN,
+				"Failed to serialize manifest!");
+		goto out;
+	}
+
+	sig = cms_sign(content, FALSE, certfile, keyfile, interfiles, &ierror);
+	if (sig == NULL) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+out:
+	return sig;
+}
+
+gboolean cms_verify_fd(gint fd, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
 {
 	GError *ierror = NULL;
 	g_autoptr(GMappedFile) file = NULL;
 	g_autoptr(GBytes) content = NULL;
 	gboolean res = FALSE;
 
-	g_return_val_if_fail(filename != NULL, FALSE);
+	g_return_val_if_fail(fd >= 0, FALSE);
 	g_return_val_if_fail(sig != NULL, FALSE);
 	g_return_val_if_fail(store != NULL, FALSE);
 	g_return_val_if_fail(cms == NULL || *cms == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	file = g_mapped_file_new(filename, FALSE, &ierror);
+	file = g_mapped_file_new_from_fd(fd, FALSE, &ierror);
 	if (file == NULL) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1023,7 +1313,27 @@ gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509
 		content = tmp;
 	}
 
-	res = cms_verify(content, sig, store, cms, &ierror);
+	res = cms_verify_bytes(content, sig, store, cms, NULL, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+out:
+	return res;
+}
+
+gboolean cms_verify_sig(GBytes *sig, X509_STORE *store, CMS_ContentInfo **cms, GBytes **manifest, GError **error)
+{
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+
+	g_return_val_if_fail(sig != NULL, FALSE);
+	g_return_val_if_fail(store != NULL, FALSE);
+	g_return_val_if_fail(cms == NULL || *cms == NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	res = cms_verify_bytes(NULL, sig, store, cms, manifest, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
