@@ -15,6 +15,7 @@
 #include "update_handler.h"
 #include "emmc.h"
 #include "mbr.h"
+#include "gpt.h"
 #include "utils.h"
 
 
@@ -47,7 +48,7 @@ static GUnixOutputStream* open_slot_device(RaucSlot *slot, int *fd, GError **err
 		return NULL;
 	}
 
-	outstream = (GUnixOutputStream *) g_unix_output_stream_new(fd_out, TRUE);
+	outstream = G_UNIX_OUTPUT_STREAM(g_unix_output_stream_new(fd_out, TRUE));
 	if (outstream == NULL) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed to open file for writing: ");
@@ -68,7 +69,7 @@ static gboolean clear_slot(RaucSlot *slot, GError **error)
 	g_autoptr(GOutputStream) outstream = NULL;
 	gint write_count = 0;
 
-	outstream = (GOutputStream *) open_slot_device(slot, NULL, &ierror);
+	outstream = G_OUTPUT_STREAM(open_slot_device(slot, NULL, &ierror));
 	if (outstream == NULL) {
 		g_propagate_error(error, ierror);
 		return FALSE;
@@ -98,6 +99,72 @@ static gboolean clear_slot(RaucSlot *slot, GError **error)
 }
 #endif
 
+/**
+ * Clear the the memory area defined in dest_partition.
+ *
+ * @param device dev path (/dev/mmcblkX)
+ * @param dest_partition partition to be cleared (start & size) *
+ * @param error return location for a GError, or NULL
+ *
+ * @return True if succeeded, False if failed
+ */
+static gboolean clear_boot_switch_partition(const gchar *device,
+		const struct boot_switch_partition *dest_partition,
+		GError **error)
+{
+	gboolean res = FALSE;
+	static gchar zerobuf[512] = {};
+	gint clear_size = sizeof(zerobuf);
+	guint clear_count = 0;
+	gint tmp_count = 0;
+	gint fd;
+
+	g_return_val_if_fail(device, FALSE);
+	g_return_val_if_fail(dest_partition, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	fd = g_open(device, O_RDWR);
+	if (fd == -1) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Opening device failed: %s",
+				g_strerror(errno));
+		goto out;
+	}
+
+	if (lseek(fd, dest_partition->start, SEEK_SET) !=
+	    (off_t)dest_partition->start) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Failed to set file to position %"G_GUINT64_FORMAT ": %s",
+				dest_partition->start, g_strerror(errno));
+		goto out;
+	}
+
+	while (clear_count < dest_partition->size) {
+		if ((dest_partition->size - clear_count) < sizeof(zerobuf))
+			clear_size = dest_partition->size - clear_count;
+
+		tmp_count = write(fd, zerobuf, clear_size);
+
+		if (tmp_count < 0)
+			break;
+
+		clear_count += tmp_count;
+	}
+
+	if (clear_count != dest_partition->size) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Failed to clear partition: %s",
+				g_strerror(errno));
+		goto out;
+	}
+	res = TRUE;
+out:
+	if (fd >= 0)
+		g_close(fd, NULL);
+
+	return res;
+}
+
 static gboolean ubifs_ioctl(RaucImage *image, int fd, GError **error)
 {
 	int ret;
@@ -122,7 +189,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 	g_autoptr(GFile) srcimagefile = g_file_new_for_path(image->filename);
 	int out_fd = g_unix_output_stream_get_fd(outstream);
 
-	g_autoptr(GInputStream) instream = (GInputStream*)g_file_read(srcimagefile, NULL, &ierror);
+	g_autoptr(GInputStream) instream = G_INPUT_STREAM(g_file_read(srcimagefile, NULL, &ierror));
 	if (instream == NULL) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed to open file for reading: ");
@@ -132,7 +199,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 	/* Do not close fd automatically to give us the chance to call fsync() on it before closing */
 	g_unix_output_stream_set_close_fd(outstream, FALSE);
 
-	size = g_output_stream_splice((GOutputStream *) outstream, instream,
+	size = g_output_stream_splice(G_OUTPUT_STREAM(outstream), instream,
 			G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
 			NULL,
 			&ierror);
@@ -142,7 +209,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 		return FALSE;
 	}
 
-	seeksize = g_seekable_tell((GSeekable*) instream);
+	seeksize = g_seekable_tell(G_SEEKABLE(instream));
 
 	if (seeksize != (goffset)image->checksum.size) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
@@ -454,6 +521,7 @@ static gboolean ext4_format_slot(RaucSlot *dest_slot, GError **error)
 		g_ptr_array_add(args, g_strdup("-L"));
 		g_ptr_array_add(args, g_strdup(dest_slot->name));
 	}
+	g_ptr_array_add(args, g_strdup("-I256"));
 	g_ptr_array_add(args, g_strdup(dest_slot->device));
 	g_ptr_array_add(args, NULL);
 
@@ -670,7 +738,11 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_CLASS", slot->sclass, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_TYPE", slot->type, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_DEVICE", slot->device, TRUE);
-	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_BOOTNAME", slot->bootname ?: "", TRUE);
+	if (slot->parent) {
+		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_BOOTNAME", slot->parent->bootname ?: "", TRUE);
+	} else {
+		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_BOOTNAME", slot->bootname ?: "", TRUE);
+	}
 	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_PARENT", slot->parent ? slot->parent->name : "", TRUE);
 	if (slot->mount_point) {
 		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_MOUNT_POINT", slot->mount_point, TRUE);
@@ -751,6 +823,7 @@ static gboolean mount_and_run_slot_hook(const gchar *hook_name, const gchar *hoo
 
 	/* finally umount slot */
 	g_message("Unmounting slot %s", slot->device);
+	ierror = NULL; /* any previous error was propagated already */
 	if (!r_umount_slot(slot, &ierror)) {
 		res = FALSE;
 		if (error && *error) {
@@ -944,6 +1017,7 @@ static gboolean archive_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot,
 unmount_out:
 	/* finally umount ubi volume */
 	g_message("Unmounting ubifs slot %s", dest_slot->device);
+	ierror = NULL; /* any previous error was propagated already */
 	if (!r_umount_slot(dest_slot, &ierror)) {
 		res = FALSE;
 		if (error && *error) {
@@ -1011,6 +1085,7 @@ static gboolean archive_to_ext4_handler(RaucImage *image, RaucSlot *dest_slot, c
 unmount_out:
 	/* finally umount ext4 volume */
 	g_message("Unmounting ext4 slot %s", dest_slot->device);
+	ierror = NULL; /* any previous error was propagated already */
 	if (!r_umount_slot(dest_slot, &ierror)) {
 		res = FALSE;
 		if (error && *error) {
@@ -1078,6 +1153,7 @@ static gboolean archive_to_vfat_handler(RaucImage *image, RaucSlot *dest_slot, c
 unmount_out:
 	/* finally umount vfat volume */
 	g_message("Unmounting vfat slot %s", dest_slot->device);
+	ierror = NULL; /* any previous error was propagated already */
 	if (!r_umount_slot(dest_slot, &ierror)) {
 		res = FALSE;
 		if (error && *error) {
@@ -1185,7 +1261,7 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 	int out_fd = -1, inactive_part;
 	g_autoptr(GUnixOutputStream) outstream = NULL;
 	GError *ierror = NULL;
-	struct mbr_switch_partition dest_partition;
+	struct boot_switch_partition dest_partition;
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
@@ -1224,7 +1300,7 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 	g_message("Clearing inactive boot partition %d on %s", inactive_part,
 			dest_slot->device);
 
-	res = r_mbr_switch_clear_partition(dest_slot->device, &dest_partition, &ierror);
+	res = clear_boot_switch_partition(dest_slot->device, &dest_partition, &ierror);
 	if (!res) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed to clear inactive partition: ");
@@ -1252,7 +1328,7 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	outstream = (GUnixOutputStream *)g_unix_output_stream_new(out_fd, FALSE);
+	outstream = G_UNIX_OUTPUT_STREAM(g_unix_output_stream_new(out_fd, FALSE));
 	if (outstream == NULL) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed to open file for writing: ");
@@ -1269,7 +1345,7 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Setting MBR to switch boot partitions");
+	g_message("Setting MBR to switch boot partition");
 
 	res = r_mbr_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
 	if (!res) {
@@ -1293,6 +1369,121 @@ out:
 	return res;
 }
 
+G_GNUC_UNUSED
+static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
+{
+	gboolean res = FALSE;
+	int out_fd = -1, inactive_part;
+	g_autoptr(GUnixOutputStream) outstream = NULL;
+	GError *ierror = NULL;
+	struct boot_switch_partition dest_partition;
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL,
+				dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	res = r_gpt_switch_get_inactive_partition(dest_slot->device,
+			&dest_partition, dest_slot->region_start,
+			dest_slot->region_size, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	if (dest_partition.start == dest_slot->region_start)
+		inactive_part = 0;
+	else
+		inactive_part = 1;
+
+	g_message("Found inactive boot partition %d (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
+			inactive_part, dest_partition.start, dest_partition.size);
+
+	if (dest_partition.size < (guint64)image->checksum.size) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Size of image (%"G_GOFFSET_FORMAT ") does not fit to slot size %"G_GUINT64_FORMAT,
+				image->checksum.size, dest_partition.size);
+		res = FALSE;
+		goto out;
+	}
+
+	g_message("Clearing inactive boot partition %d on %s", inactive_part,
+			dest_slot->device);
+
+	res = clear_boot_switch_partition(dest_slot->device, &dest_partition, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(error, ierror,
+				"Failed to clear inactive partition: ");
+		goto out;
+	}
+
+	g_message("Opening inactive boot partition %d on %s", inactive_part,
+			dest_slot->device);
+
+	out_fd = open(dest_slot->device, O_WRONLY);
+	if (out_fd == -1) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Opening output device failed: %s",
+				strerror(errno));
+		res = FALSE;
+		goto out;
+	}
+
+	if (lseek(out_fd, dest_partition.start, SEEK_SET) !=
+	    (off_t)dest_partition.start) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Failed to set file to position %"G_GUINT64_FORMAT,
+				dest_partition.start);
+		res = FALSE;
+		goto out;
+	}
+
+	outstream = G_UNIX_OUTPUT_STREAM(g_unix_output_stream_new(out_fd, FALSE));
+	if (outstream == NULL) {
+		g_propagate_prefixed_error(error, ierror,
+				"Failed to open file for writing: ");
+		res = FALSE;
+		goto out;
+	}
+
+	g_message("Copying image to inactive boot partition %d on %s",
+			inactive_part, dest_slot->device);
+
+	res = copy_raw_image(image, outstream, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	g_message("Setting GPT to switch boot partition");
+
+	res = r_gpt_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* run slot post install hook if enabled */
+	if (hook_name && image->hooks.post_install) {
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL,
+				dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+out:
+	if (out_fd >= 0)
+		close(out_fd);
+
+	return res;
+}
+
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
@@ -1527,7 +1718,6 @@ RaucUpdatePair updatepairs[] = {
 	{"*.img.caibx", "ubivol", img_to_ubivol_handler},
 	{"*.img.caibx", "ubifs", img_to_ubifs_handler},
 	{"*.squashfs.caibx", "ubivol", img_to_ubivol_handler},
-	{"*.img.caibx", "*", img_to_raw_handler}, /* fallback */
 	{"*.caidx", "ext4", archive_to_ext4_handler},
 	{"*.caidx", "ubifs", archive_to_ubifs_handler},
 	{"*.caidx", "vfat", archive_to_vfat_handler},
@@ -1550,8 +1740,18 @@ RaucUpdatePair updatepairs[] = {
 	{"*.squashfs", "ubivol", img_to_ubivol_handler},
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 	{"*.img", "boot-emmc", img_to_boot_emmc_handler},
+	{"*", "boot-emmc", NULL},
 #endif
 	{"*.vfat", "boot-mbr-switch", img_to_boot_mbr_switch_handler},
+	{"*.img", "boot-mbr-switch", img_to_boot_mbr_switch_handler},
+	{"*", "boot-mbr-switch", NULL},
+#if ENABLE_GPT == 1
+	{"*.vfat", "boot-gpt-switch", img_to_boot_gpt_switch_handler},
+	{"*.ext4", "boot-gpt-switch", img_to_boot_gpt_switch_handler},
+	{"*.img", "boot-gpt-switch", img_to_boot_gpt_switch_handler},
+#endif
+	{"*", "boot-gpt-switch", NULL},
+	{"*.img.caibx", "*", img_to_raw_handler}, /* fallback */
 	{"*.img", "*", img_to_raw_handler}, /* fallback */
 	{0}
 };
@@ -1569,7 +1769,7 @@ img_to_slot_handler get_update_handler(RaucImage *mfimage, RaucSlot *dest_slot,
 
 	g_message("Checking image type for slot type: %s", dest);
 
-	for (RaucUpdatePair *updatepair = updatepairs; updatepair->handler != NULL; updatepair++) {
+	for (RaucUpdatePair *updatepair = updatepairs; updatepair->src != NULL; updatepair++) {
 		if (g_pattern_match_simple(updatepair->src, src) &&
 		    g_pattern_match_simple(updatepair->dest, dest)) {
 			g_message("Image detected as type: %s", updatepair->src);
