@@ -18,6 +18,11 @@ typedef struct {
 	gchar *contentdir;
 } BundleFixture;
 
+typedef struct {
+	ManifestTestOptions manifest_test_options;
+	const gchar *bundle_formats;
+} BundleData;
+
 static void bundle_fixture_set_up(BundleFixture *fixture,
 		gconstpointer user_data)
 {
@@ -26,9 +31,15 @@ static void bundle_fixture_set_up(BundleFixture *fixture,
 	g_print("bundle tmpdir: %s\n", fixture->tmpdir);
 }
 
-static void bundle_fixture_set_up_bundle(BundleFixture *fixture,
-		gconstpointer user_data)
+static void prepare_bundle(BundleFixture *fixture, gconstpointer user_data)
 {
+	BundleData *data = (BundleData*)user_data;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+
+	/* the context needs to be setup before calling this */
+	r_context();
+
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 	g_assert_nonnull(fixture->tmpdir);
 
@@ -37,9 +48,29 @@ static void bundle_fixture_set_up_bundle(BundleFixture *fixture,
 	fixture->bundlename = g_build_filename(fixture->tmpdir, "bundle.raucb", NULL);
 	g_assert_nonnull(fixture->bundlename);
 
-	test_create_content(fixture->contentdir);
+	test_create_content(fixture->contentdir, &data->manifest_test_options);
 
+	/* disable crl checking during bundle creation */
+	r_context()->config->keyring_check_crl = FALSE;
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+			"Detected CRL but CRL checking is disabled!");
 	test_create_bundle(fixture->contentdir, fixture->bundlename);
+	r_context()->config->keyring_check_crl = TRUE;
+
+	if (data->bundle_formats) {
+		res = parse_bundle_formats(&r_context()->config->bundle_formats_mask, data->bundle_formats, &ierror);
+		g_assert_no_error(ierror);
+		g_assert_true(res);
+	}
+}
+
+static void bundle_fixture_set_up_bundle(BundleFixture *fixture,
+		gconstpointer user_data)
+{
+	r_context_conf()->certpath = g_strdup("test/openssl-ca/dev/autobuilder-1.cert.pem");
+	r_context_conf()->keypath = g_strdup("test/openssl-ca/dev/private/autobuilder-1.pem");
+
+	prepare_bundle(fixture, user_data);
 }
 
 static void bundle_fixture_set_up_bundle_autobuilder2(BundleFixture *fixture,
@@ -48,12 +79,7 @@ static void bundle_fixture_set_up_bundle_autobuilder2(BundleFixture *fixture,
 	r_context_conf()->certpath = g_strdup("test/openssl-ca/dev/autobuilder-2.cert.pem");
 	r_context_conf()->keypath = g_strdup("test/openssl-ca/dev/private/autobuilder-2.pem");
 
-	/* disable crl checking during bundle creation */
-	r_context()->config->keyring_check_crl = FALSE;
-	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
-			"Detected CRL but CRL checking is disabled!");
-	bundle_fixture_set_up_bundle(fixture, user_data);
-	r_context()->config->keyring_check_crl = TRUE;
+	prepare_bundle(fixture, user_data);
 }
 
 static void bundle_fixture_set_up_bundle_email(BundleFixture *fixture,
@@ -64,7 +90,7 @@ static void bundle_fixture_set_up_bundle_email(BundleFixture *fixture,
 	/* cert is already checked once during signing */
 	r_context()->config->keyring_check_purpose = g_strdup("smimesign");
 
-	bundle_fixture_set_up_bundle(fixture, user_data);
+	prepare_bundle(fixture, user_data);
 }
 
 static void bundle_fixture_set_up_bundle_codesign(BundleFixture *fixture,
@@ -75,7 +101,7 @@ static void bundle_fixture_set_up_bundle_codesign(BundleFixture *fixture,
 	/* cert is already checked once during signing */
 	r_context()->config->keyring_check_purpose = g_strdup("codesign");
 
-	bundle_fixture_set_up_bundle(fixture, user_data);
+	prepare_bundle(fixture, user_data);
 }
 
 static void bundle_fixture_tear_down(BundleFixture *fixture,
@@ -83,24 +109,22 @@ static void bundle_fixture_tear_down(BundleFixture *fixture,
 {
 	g_assert_true(rm_tree(fixture->tmpdir, NULL));
 	g_free(fixture->tmpdir);
+
+	g_test_assert_expected_messages();
 }
 
 static void bundle_fixture_tear_down_autobuilder2(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	/* restore old paths */
-	r_context_conf()->certpath = g_strdup("test/openssl-ca/dev/autobuilder-1.cert.pem");
-	r_context_conf()->keypath = g_strdup("test/openssl-ca/dev/private/autobuilder-1.pem");
-
 	bundle_fixture_tear_down(fixture, user_data);
 }
 
 static void test_check_empty_bundle(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlename;
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autofree gchar *bundlename = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	bundlename = write_random_file(fixture->tmpdir, "bundle.raucb", 0, 1234);
@@ -110,16 +134,14 @@ static void test_check_empty_bundle(BundleFixture *fixture,
 	g_assert_false(res);
 	g_assert_error(ierror, G_IO_ERROR, G_IO_ERROR_PARTIAL_INPUT);
 	g_assert_null(bundle);
-
-	g_free(bundlename);
 }
 
 static void test_check_invalid_bundle(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlename;
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autofree gchar *bundlename = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	bundlename = write_random_file(fixture->tmpdir, "bundle.raucb", 1024, 1234);
@@ -129,32 +151,60 @@ static void test_check_invalid_bundle(BundleFixture *fixture,
 	g_assert_false(res);
 	g_assert_error(ierror, R_BUNDLE_ERROR, R_BUNDLE_ERROR_IDENTIFIER);
 	g_assert_null(bundle);
+}
 
-	g_free(bundlename);
+static void bundle_test_create_check_error(BundleFixture *fixture,
+		gconstpointer user_data)
+{
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+
+	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	g_assert_false(res);
+	g_assert_error(ierror, R_BUNDLE_ERROR, R_BUNDLE_ERROR_FORMAT);
+	g_assert_null(bundle);
 }
 
 static void bundle_test_create_extract(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *outputdir;
-	RaucBundle *bundle = NULL;
+	g_autofree gchar *outputdir = NULL;
+	g_autofree gchar *filepath = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
 
 	outputdir = g_build_filename(fixture->tmpdir, "output", NULL);
 	g_assert_nonnull(outputdir);
 
-	g_assert_true(check_bundle(fixture->bundlename, &bundle, TRUE, NULL));
+	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(bundle);
 
-	g_assert_true(extract_bundle(bundle, outputdir, NULL));
+	res = extract_bundle(bundle, outputdir, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 
-	free_bundle(bundle);
+	filepath = g_build_filename(outputdir, "manifest.raucm", NULL);
+	g_assert_true(g_file_test(filepath, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&filepath, g_free);
+
+	filepath = g_build_filename(outputdir, "rootfs.ext4", NULL);
+	g_assert_true(g_file_test(filepath, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&filepath, g_free);
+
+	filepath = g_build_filename(outputdir, "appfs.ext4", NULL);
+	g_assert_true(g_file_test(filepath, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&filepath, g_free);
 }
 
 static void bundle_test_create_mount_extract(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	/* mount needs to run as root */
@@ -173,29 +223,119 @@ static void bundle_test_create_mount_extract(BundleFixture *fixture,
 	res = umount_bundle(bundle, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
-
-	free_bundle(bundle);
 }
 
 static void bundle_test_extract_manifest(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *outputdir, *manifestpath;
-	RaucBundle *bundle = NULL;
+	g_autofree gchar *outputdir = NULL;
+	g_autofree gchar *filepath = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
 
 	outputdir = g_build_filename(fixture->tmpdir, "output", NULL);
 	g_assert_nonnull(outputdir);
 
-	manifestpath = g_build_filename(fixture->tmpdir, "/output/manifest.raucm", NULL);
-	g_assert_nonnull(manifestpath);
-
-	g_assert_true(check_bundle(fixture->bundlename, &bundle, TRUE, NULL));
+	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(bundle);
 
-	g_assert_true(extract_file_from_bundle(bundle, outputdir, "manifest.raucm", NULL));
-	g_assert_true(g_file_test(manifestpath, G_FILE_TEST_EXISTS));
+	res = extract_file_from_bundle(bundle, outputdir, "manifest.raucm", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 
-	free_bundle(bundle);
+	filepath = g_build_filename(outputdir, "manifest.raucm", NULL);
+	g_assert_true(g_file_test(filepath, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&filepath, g_free);
+
+	filepath = g_build_filename(outputdir, "rootfs.ext4", NULL);
+	g_assert_false(g_file_test(filepath, G_FILE_TEST_EXISTS));
+	g_clear_pointer(&filepath, g_free);
+
+	filepath = g_build_filename(outputdir, "appfs.ext4", NULL);
+	g_assert_false(g_file_test(filepath, G_FILE_TEST_EXISTS));
+	g_clear_pointer(&filepath, g_free);
+}
+
+
+static void assert_casync_manifest(RaucManifest *rm)
+{
+	RaucImage *test_img = NULL;
+
+	g_assert_cmpuint(g_list_length(rm->images), ==, 2);
+
+	test_img = g_list_nth_data(rm->images, 0);
+	g_assert_nonnull(test_img);
+	g_assert_cmpstr(test_img->filename, ==, "rootfs.img.caibx");
+	g_assert_cmpstr(test_img->checksum.digest, ==, "de2f256064a0af797747c2b97505dc0b9f3df0de4f489eac731c23ae9ca9cc31");
+	g_assert_cmpuint(test_img->checksum.size, ==, 65536);
+
+	test_img = g_list_nth_data(rm->images, 1);
+	g_assert_nonnull(test_img);
+	g_assert_cmpstr(test_img->filename, ==, "appfs.img.caibx");
+	g_assert_cmpstr(test_img->checksum.digest, ==, "c35020473aed1b4642cd726cad727b63fff2824ad68cedd7ffb73c7cbd890479");
+	g_assert_cmpuint(test_img->checksum.size, ==, 32768);
+}
+
+static void bundle_test_check_casync_old(BundleFixture *fixture, gconstpointer user_data)
+{
+	g_autofree gchar *bundlepath = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+
+	bundlepath = g_build_filename(fixture->tmpdir, "bundle.raucb", NULL);
+	g_assert_true(test_copy_file("test", "good-casync-bundle-1.4.raucb", fixture->tmpdir, "bundle.raucb"));
+
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"Reading bundle*" );
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"Payload size (5661) is not a multiple of 4KiB.*");
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"Verifying bundle signature*" );
+
+	res = check_bundle(bundlepath, &bundle, TRUE, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(bundle);
+
+	res = load_manifest_from_bundle(bundle, &bundle->manifest, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(bundle->manifest);
+
+
+	assert_casync_manifest(bundle->manifest);
+}
+
+static void bundle_test_check_casync_new(BundleFixture *fixture, gconstpointer user_data)
+{
+	g_autofree gchar *bundlepath = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+
+	bundlepath = g_build_filename(fixture->tmpdir, "bundle.raucb", NULL);
+	g_assert_true(test_copy_file("test", "good-casync-bundle-1.5.1.raucb", fixture->tmpdir, "bundle.raucb"));
+
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"Reading bundle*");
+	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"Verifying bundle signature*" );
+
+	res = check_bundle(bundlepath, &bundle, TRUE, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(bundle);
+
+	res = load_manifest_from_bundle(bundle, &bundle->manifest, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(bundle->manifest);
+
+	assert_casync_manifest(bundle->manifest);
 }
 
 // Hack to pull-in context for testing modification
@@ -204,9 +344,9 @@ extern RaucContext *context;
 static void bundle_test_resign(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *resignbundle;
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autofree gchar *resignbundle = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	resignbundle = g_build_filename(fixture->tmpdir, "resigned-bundle.raucb", NULL);
@@ -262,14 +402,13 @@ static void bundle_test_resign(BundleFixture *fixture,
 
 	// hacky restore of original signing_keyringpath
 	context->signing_keyringpath = NULL;
-	g_clear_pointer(&bundle, free_bundle);
 }
 
 static void bundle_test_wrong_capath(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	r_context()->config->keyring_path = g_strdup("does/not/exist.pem");
 
 	g_assert_false(check_bundle(fixture->bundlename, &bundle, TRUE, &ierror));
@@ -285,8 +424,8 @@ static void bundle_test_wrong_capath(BundleFixture *fixture,
 static void bundle_test_verify_no_crl_warn(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	r_context()->config->keyring_check_crl = FALSE;
@@ -299,8 +438,6 @@ static void bundle_test_verify_no_crl_warn(BundleFixture *fixture,
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
-
-	free_bundle(bundle);
 }
 
 /* Test that verification of a bundle signed with a revoked key actually fails
@@ -308,8 +445,8 @@ static void bundle_test_verify_no_crl_warn(BundleFixture *fixture,
 static void bundle_test_verify_revoked(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 
 	g_assert_false(check_bundle(fixture->bundlename, &bundle, TRUE, &ierror));
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
@@ -319,8 +456,8 @@ static void bundle_test_verify_revoked(BundleFixture *fixture,
 static void bundle_test_purpose_default(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	/* When the cert specifies no purpose, everything except 'codesign' is allowed */
@@ -362,8 +499,8 @@ static void bundle_test_purpose_default(BundleFixture *fixture,
 static void bundle_test_purpose_email(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	/* When the cert specifies the 'smimesign' usage, only default and that is allowed */
@@ -406,8 +543,8 @@ static void bundle_test_purpose_email(BundleFixture *fixture,
 static void bundle_test_purpose_codesign(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucBundle *bundle = NULL;
-	GError *ierror = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	/* When the cert specifies the 'codesign' usage, only that is allowed */
@@ -449,61 +586,205 @@ static void bundle_test_purpose_codesign(BundleFixture *fixture,
 
 int main(int argc, char *argv[])
 {
+	BundleData *bundle_data;
 	setlocale(LC_ALL, "C");
 
 	r_context_conf()->configpath = g_strdup("test/test.conf");
-	r_context_conf()->certpath = g_strdup("test/openssl-ca/dev/autobuilder-1.cert.pem");
-	r_context_conf()->keypath = g_strdup("test/openssl-ca/dev/private/autobuilder-1.pem");
 	r_context();
 
 	g_test_init(&argc, &argv, NULL);
 
-	g_test_add("/bundle/check/empty", BundleFixture, NULL,
-			bundle_fixture_set_up, test_check_empty_bundle,
+	for (RManifestBundleFormat format = R_MANIFEST_FORMAT_PLAIN; format <= R_MANIFEST_FORMAT_VERITY; format++) {
+		const gchar *format_name = r_manifest_bundle_format_to_str(format);
+
+		bundle_data = memdup((&(BundleData) {
+			.manifest_test_options = {
+			        .format = format,
+			},
+		}));
+
+		g_test_add(g_strdup_printf("/bundle/check/empty/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up, test_check_empty_bundle,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/check/invalid/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up, test_check_invalid_bundle,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/create_extract/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_create_extract,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/create_mount_extract/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_create_mount_extract,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/extract_manifest/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_extract_manifest,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/resign/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_resign,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/wrong_capath/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_wrong_capath,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/verify_no_crl_warn/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_verify_no_crl_warn,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/verify_revoked/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle_autobuilder2, bundle_test_verify_revoked,
+				bundle_fixture_tear_down_autobuilder2);
+
+		g_test_add(g_strdup_printf("/bundle/purpose/default/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_purpose_default,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/purpose/email/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle_email, bundle_test_purpose_email,
+				bundle_fixture_tear_down);
+
+		g_test_add(g_strdup_printf("/bundle/purpose/codesign/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle_codesign, bundle_test_purpose_codesign,
+				bundle_fixture_tear_down);
+	}
+
+	/* test casync manifest contents */
+	g_test_add(g_strdup_printf("/bundle/check_casync/old"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up, bundle_test_check_casync_old,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/check/invalid", BundleFixture, NULL,
-			bundle_fixture_set_up, test_check_invalid_bundle,
+	g_test_add(g_strdup_printf("/bundle/check_casync/new"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up, bundle_test_check_casync_new,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/create_extract", BundleFixture, NULL,
+	/* test plain bundles against possible masks */
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_PLAIN,
+		},
+		.bundle_formats = "plain",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/plain/set-plain"),
+			BundleFixture, bundle_data,
 			bundle_fixture_set_up_bundle, bundle_test_create_extract,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/create_mount_extract", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_create_mount_extract,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_PLAIN,
+		},
+		.bundle_formats = "verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/plain/set-verity"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_check_error,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/extract_manifest", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_extract_manifest,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_PLAIN,
+		},
+		.bundle_formats = "plain verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/plain/set-both"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_extract,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/resign", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_resign,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_PLAIN,
+		},
+		.bundle_formats = "-plain",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/plain/deny-plain"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_check_error,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/wrong_capath", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_wrong_capath,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_PLAIN,
+		},
+		.bundle_formats = "-verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/plain/deny-verity"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_extract,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/verify_no_crl_warn", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_verify_no_crl_warn,
+	/* test verity bundles against possible masks */
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		},
+		.bundle_formats = "plain",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/verity/set-plain"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_check_error,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/verify_revoked", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle_autobuilder2, bundle_test_verify_revoked,
-			bundle_fixture_tear_down_autobuilder2);
-
-	g_test_add("/bundle/purpose/default", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle, bundle_test_purpose_default,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		},
+		.bundle_formats = "verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/verity/set-verity"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_extract,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/purpose/email", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle_email, bundle_test_purpose_email,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		},
+		.bundle_formats = "plain verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/verity/set-both"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_extract,
 			bundle_fixture_tear_down);
 
-	g_test_add("/bundle/purpose/codesign", BundleFixture, NULL,
-			bundle_fixture_set_up_bundle_codesign, bundle_test_purpose_codesign,
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		},
+		.bundle_formats = "-plain",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/verity/deny-plain"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_extract,
+			bundle_fixture_tear_down);
+
+	bundle_data = memdup((&(BundleData) {
+		.manifest_test_options = {
+		        .format = R_MANIFEST_FORMAT_VERITY,
+		},
+		.bundle_formats = "-verity",
+	}));
+	g_test_add(g_strdup_printf("/bundle/format/verity/deny-verity"),
+			BundleFixture, bundle_data,
+			bundle_fixture_set_up_bundle, bundle_test_create_check_error,
 			bundle_fixture_tear_down);
 
 	return g_test_run();
