@@ -46,6 +46,8 @@ static void config_file_fixture_tear_down(ConfigFileFixture *fixture,
 static void config_file_full_config(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
+	GError *ierror = NULL;
+	gboolean res;
 	GList *slotlist;
 	RaucConfig *config;
 	RaucSlot *slot;
@@ -58,6 +60,7 @@ bootloader=barebox\n\
 mountprefix=/mnt/myrauc/\n\
 statusfile=/mnt/persistent-rw-fs/system.raucs\n\
 max-bundle-download-size=42\n\
+bundle-formats=verity\n\
 \n\
 [keyring]\n\
 path=/etc/rauc/keyring/\n\
@@ -106,7 +109,9 @@ install-same=false\n";
 	gchar* pathname = write_tmp_file(fixture->tmpdir, "full_config.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, NULL));
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_cmpstr(config->system_compatible, ==, "FooCorp Super BarBazzer");
 	g_assert_cmpstr(config->system_bootloader, ==, "barebox");
@@ -114,6 +119,7 @@ install-same=false\n";
 	g_assert_true(config->activate_installed);
 	g_assert_cmpstr(config->statusfile_path, ==, "/mnt/persistent-rw-fs/system.raucs");
 	g_assert_cmpint(config->max_bundle_download_size, ==, 42);
+	g_assert_cmphex(config->bundle_formats_mask, ==, 0x2);
 
 	g_assert_nonnull(config->slots);
 	slotlist = g_hash_table_get_keys(config->slots);
@@ -285,6 +291,127 @@ parent=invalid\n\
 	g_clear_error(&ierror);
 }
 
+static void config_file_parent_has_parent(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	RaucConfig *config;
+	RaucSlot *parentslot;
+	RaucSlot *childslot;
+	RaucSlot *grandchildslot;
+	gchar* pathname;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/null\n\
+\n\
+[slot.child.0]\n\
+device=/dev/null\n\
+parent=rootfs.0\n\
+\n\
+[slot.grandchild.0]\n\
+device=/dev/null\n\
+parent=child.0\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "parent_has_parent.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_true(load_config(pathname, &config, NULL));
+	g_assert_nonnull(config);
+
+	parentslot = g_hash_table_lookup(config->slots, "rootfs.0");
+	childslot = g_hash_table_lookup(config->slots, "child.0");
+	g_assert(childslot->parent == parentslot);
+	grandchildslot = g_hash_table_lookup(config->slots, "grandchild.0");
+	g_assert(grandchildslot->parent == parentslot);
+}
+
+static void config_file_parent_loop(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	RaucConfig *config;
+	GError *ierror = NULL;
+	gchar* pathname;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/null\n\
+parent=child.0\n\
+\n\
+[slot.child.0]\n\
+device=/dev/null\n\
+parent=rootfs.0\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "parent_loop.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_PARENT_LOOP);
+	g_clear_error(&ierror);
+}
+
+static void config_file_bootname_set_on_child(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	RaucConfig *config;
+	GError *ierror = NULL;
+	gchar* pathname;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.parent.0]\n\
+device=/dev/null\n\
+bootname=slot0\n\
+\n\
+[slot.child.0]\n\
+device=/dev/null\n\
+parent=parent.0\n\
+bootname=slotchild0\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "bootname_set_on_child.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_CHILD_HAS_BOOTNAME);
+	g_assert_cmpstr(ierror->message, ==, "Child slot 'child.0' has bootname set");
+	g_clear_error(&ierror);
+}
+
+static void config_file_duplicate_bootname(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	RaucConfig *config;
+	GError *ierror = NULL;
+	gchar* pathname;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/null\n\
+bootname=theslot\n\
+\n\
+[slot.rootfs.1]\n\
+device=/dev/null\n\
+bootname=theslot\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "duplicate_bootname.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_DUPLICATE_BOOTNAME);
+	g_assert_cmpstr(ierror->message, ==, "Bootname 'theslot' is set on more than one slot");
+	g_clear_error(&ierror);
+}
+
 static void config_file_typo(ConfigFileFixture *fixture, const gchar *cfg_file)
 {
 	RaucConfig *config;
@@ -417,6 +544,7 @@ static void config_file_no_max_bundle_download_size(ConfigFileFixture *fixture,
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	gchar* pathname;
 
 	const gchar *cfg_file = "\
@@ -427,8 +555,9 @@ bootloader=barebox\n";
 	pathname = write_tmp_file(fixture->tmpdir, "no_max_bundle_download_size.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_cmpuint(config->max_bundle_download_size, ==, DEFAULT_MAX_BUNDLE_DOWNLOAD_SIZE);
 
@@ -473,6 +602,7 @@ static void config_file_activate_installed_set_to_true(ConfigFileFixture *fixtur
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	gchar* pathname;
 
 	const gchar *cfg_file = "\
@@ -486,8 +616,9 @@ activate-installed=true\n";
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_true(config->activate_installed);
 
@@ -499,6 +630,7 @@ static void config_file_activate_installed_set_to_false(ConfigFileFixture *fixtu
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	gchar* pathname;
 
 	const gchar *cfg_file = "\
@@ -512,8 +644,9 @@ activate-installed=false\n";
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_false(config->activate_installed);
 
@@ -525,6 +658,7 @@ static void config_file_system_variant(ConfigFileFixture *fixture,
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	gchar* pathname;
 
 	const gchar *cfg_file_no_variant = "\
@@ -565,7 +699,9 @@ variant-name=xxx";
 	pathname = write_tmp_file(fixture->tmpdir, "no_variant.conf", cfg_file_no_variant, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_free(pathname);
 	g_assert_null(ierror);
 	g_assert_nonnull(config);
@@ -576,7 +712,9 @@ variant-name=xxx";
 	pathname = write_tmp_file(fixture->tmpdir, "name_variant.conf", cfg_file_name_variant, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_free(pathname);
 	g_assert_null(ierror);
 	g_assert_nonnull(config);
@@ -588,7 +726,9 @@ variant-name=xxx";
 	pathname = write_tmp_file(fixture->tmpdir, "dtb_variant.conf", cfg_file_dtb_variant, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_free(pathname);
 	g_assert_null(ierror);
 	g_assert_nonnull(config);
@@ -600,7 +740,9 @@ variant-name=xxx";
 	pathname = write_tmp_file(fixture->tmpdir, "file_variant.conf", cfg_file_file_variant, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_free(pathname);
 	g_assert_null(ierror);
 	g_assert_nonnull(config);
@@ -621,6 +763,7 @@ static void config_file_no_extra_mount_opts(ConfigFileFixture *fixture,
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	g_autofree gchar* pathname = NULL;
 	RaucSlot *slot = NULL;
 
@@ -638,8 +781,9 @@ device=/dev/null\n";
 	pathname = write_tmp_file(fixture->tmpdir, "extra_mount.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 
 
@@ -656,6 +800,7 @@ static void config_file_extra_mount_opts(ConfigFileFixture *fixture,
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	g_autofree gchar* pathname = NULL;
 	RaucSlot *slot = NULL;
 
@@ -674,8 +819,9 @@ extra-mount-opts=ro,noatime\n";
 	pathname = write_tmp_file(fixture->tmpdir, "extra_mount.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
 
 	slot = g_hash_table_lookup(config->slots, "rootfs.0");
@@ -690,6 +836,7 @@ static void config_file_statusfile_missing(ConfigFileFixture *fixture,
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
+	gboolean res;
 	gchar* pathname;
 
 	const gchar *cfg_file = "\
@@ -702,10 +849,12 @@ mountprefix=/mnt/myrauc/\n";
 	pathname = write_tmp_file(fixture->tmpdir, "valid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
-	g_assert_true(load_config(pathname, &config, &ierror));
-	g_assert_null(ierror);
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(config);
-	g_assert_null(config->statusfile_path);
+	g_assert_nonnull(config->statusfile_path);
+	g_assert_cmpstr(config->statusfile_path, ==, "per-slot");
 
 	free_config(config);
 }
@@ -713,8 +862,12 @@ mountprefix=/mnt/myrauc/\n";
 
 static void config_file_test_read_slot_status(void)
 {
+	GError *ierror = NULL;
+	gboolean res;
 	RaucSlotStatus *ss = g_new0(RaucSlotStatus, 1);
-	g_assert_true(read_slot_status("test/rootfs.raucs", ss, NULL));
+	res = read_slot_status("test/rootfs.raucs", ss, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
 	g_assert_nonnull(ss);
 	g_assert_cmpstr(ss->status, ==, "ok");
 	g_assert_cmpint(ss->checksum.type, ==, G_CHECKSUM_SHA256);
@@ -733,12 +886,12 @@ static void config_file_test_write_slot_status(void)
 	ss->checksum.type = G_CHECKSUM_SHA256;
 	ss->checksum.digest = g_strdup("dc626520dcd53a22f727af3ee42c770e56c97a64fe3adb063799d8ab032fe551");
 
-	write_slot_status("test/savedslot.raucs", ss, NULL);
+	g_assert_true(write_slot_status("test/savedslot.raucs", ss, NULL));
 
 	r_slot_free_status(ss);
 	ss = g_new0(RaucSlotStatus, 1);
 
-	read_slot_status("test/savedslot.raucs", ss, NULL);
+	g_assert_true(read_slot_status("test/savedslot.raucs", ss, NULL));
 
 	g_assert_nonnull(ss);
 	g_assert_cmpstr(ss->status, ==, "ok");
@@ -763,6 +916,7 @@ static void config_file_test_global_slot_status(ConfigFileFixture *fixture,
 	GHashTableIter iter;
 	GError *ierror = NULL;
 	RaucSlot *slot;
+	gboolean res;
 
 	g_assert_nonnull(r_context()->config->statusfile_path);
 
@@ -782,8 +936,9 @@ static void config_file_test_global_slot_status(ConfigFileFixture *fixture,
 	/* Save status for all slots */
 	g_hash_table_iter_init(&iter, slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
-		save_slot_status(slot, &ierror);
+		res = save_slot_status(slot, &ierror);
 		g_assert_no_error(ierror);
+		g_assert_true(res);
 	}
 
 	/* Clear status for all slots */
@@ -855,6 +1010,131 @@ check-purpose=codesign\n";
 	free_config(config);
 }
 
+static void config_file_bundle_formats(ConfigFileFixture *fixture,
+		gconstpointer user_data)
+{
+	RaucConfig *config;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+	gchar* pathname;
+
+	const gchar *default_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n";
+	const gchar *set_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+bundle-formats=plain\n";
+	const gchar *modify_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+bundle-formats=-plain\n";
+	const gchar *none_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+bundle-formats=-plain -verity\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "default.conf", default_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(config);
+	g_assert_cmphex(config->bundle_formats_mask, ==, 0x3);
+
+	free_config(config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "set.conf", set_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(config);
+	g_assert_cmphex(config->bundle_formats_mask, ==, 0x1);
+
+	free_config(config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "modify.conf", modify_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(config);
+	g_assert_cmphex(config->bundle_formats_mask, ==, 0x2);
+
+	free_config(config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "none.conf", none_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_INVALID_FORMAT);
+	g_assert_cmpstr(ierror->message, ==, "Invalid bundle format configuration '-plain -verity', no remaining formats");
+	g_assert_false(res);
+	g_assert_null(config);
+	g_clear_error(&ierror);
+}
+
+static void config_file_test_parse_bundle_formats(void)
+{
+	guint mask;
+	gboolean res;
+	g_autoptr(GError) ierror = NULL;
+
+	mask = 0x0;
+	res = parse_bundle_formats(&mask, "plain  verity", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_cmphex(mask, ==, 0x3);
+	g_assert_true(res);
+
+	mask = 0x2;
+	res = parse_bundle_formats(&mask, "+plain -verity", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_cmphex(mask, ==, 0x1);
+	g_assert_true(res);
+
+	mask = 0x3;
+	res = parse_bundle_formats(&mask, "-verity", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_cmphex(mask, ==, 0x1);
+	g_assert_true(res);
+
+	mask = 0x3;
+	res = parse_bundle_formats(&mask, "-verity +verity", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_cmphex(mask, ==, 0x3);
+	g_assert_true(res);
+
+	mask = 0x3;
+	res = parse_bundle_formats(&mask, "-verity plain", &ierror);
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_INVALID_FORMAT);
+	g_assert_cmpstr(ierror->message, ==, "Invalid bundle format configuration '-verity plain', cannot combine fixed value with modification (+/-)");
+	g_assert_cmphex(mask, ==, 0x3);
+	g_assert_false(res);
+	g_clear_error(&ierror);
+
+	mask = 0x3;
+	res = parse_bundle_formats(&mask, "", &ierror);
+	g_assert_no_error(ierror);
+	g_assert_cmphex(mask, ==, 0x3);
+	g_assert_true(res);
+
+	mask = 0x3;
+	res = parse_bundle_formats(&mask, "-verity -plain", &ierror);
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_INVALID_FORMAT);
+	g_assert_cmpstr(ierror->message, ==, "Invalid bundle format configuration '-verity -plain', no remaining formats");
+	g_assert_cmphex(mask, ==, 0x3);
+	g_assert_false(res);
+	g_clear_error(&ierror);
+}
+
 int main(int argc, char *argv[])
 {
 	setlocale(LC_ALL, "C");
@@ -873,6 +1153,18 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/invalid-parent", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_invalid_parent,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/parent-has-parent", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_parent_has_parent,
+			config_file_fixture_tear_down);
+	g_test_add("/config-file/parent-loop", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_parent_loop,
+			config_file_fixture_tear_down);
+	g_test_add("/config-file/bootname-set-on-child", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_bootname_set_on_child,
+			config_file_fixture_tear_down);
+	g_test_add("/config-file/duplicate-bootname", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_duplicate_bootname,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/typo-in-boolean-allow-mounted-key", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_typo_in_boolean_allow_mounted_key,
 			config_file_fixture_tear_down);
@@ -932,6 +1224,10 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/keyring-checks", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_keyring_checks,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/bundle-formats", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_bundle_formats,
+			config_file_fixture_tear_down);
+	g_test_add_func("/config-file/parse-bundle-formats", config_file_test_parse_bundle_formats);
 
 	return g_test_run();
 }
