@@ -21,6 +21,11 @@ case "$1" in
 		test -n "$RAUC_SLOT_NAME" || die_error "missing RAUC_SLOT_NAME"
 		test -n "$RAUC_SLOT_CLASS" || die_error "missing RAUC_SLOT_CLASS"
 		test -n "$RAUC_SLOT_TYPE" || die_error "missing RAUC_SLOT_TYPE"
+		if [ "$RAUC_SLOT_CLASS" == "bootloader" ]; then
+			echo "$NAME: no bootname expected for $RAUC_SLOT_NAME"
+		else
+			test -n "$RAUC_SLOT_BOOTNAME" || die_error "missing RAUC_SLOT_BOOTNAME for $RAUC_SLOT_NAME"
+		fi
 
 		# only rootfs needs to be handled
 		test "$RAUC_SLOT_CLASS" = "rootfs" || exit 0
@@ -34,6 +39,11 @@ case "$1" in
 		test -n "$RAUC_SLOT_NAME" || die_error "missing RAUC_SLOT_NAME"
 		test -n "$RAUC_SLOT_CLASS" || die_error "missing RAUC_SLOT_CLASS"
 		test -n "$RAUC_SLOT_TYPE" || die_error "missing RAUC_SLOT_TYPE"
+		if [ "$RAUC_SLOT_CLASS" == "bootloader" ]; then
+			echo "$NAME: no bootname expected for $RAUC_SLOT_NAME"
+		else
+			test -n "$RAUC_SLOT_BOOTNAME" || die_error "missing RAUC_SLOT_BOOTNAME for $RAUC_SLOT_NAME"
+		fi
 
 		echo "RAUC_IMAGE_PATH: $RAUC_IMAGE_PATH"
 		echo "RAUC_SLOT_DEVICE: $RAUC_SLOT_DEVICE"
