@@ -14,28 +14,35 @@
 
 GMainLoop *r_loop = NULL;
 
+typedef struct {
+	const gchar **message_needles;
+	ManifestTestOptions manifest_test_options;
+} InstallData;
 
 static void install_fixture_set_up_bundle(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, FALSE, FALSE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_central_status(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, "test/test-global.conf");
-	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, FALSE, FALSE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_already_mounted(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=Test Config\n\
@@ -51,12 +58,13 @@ hooks=install\n\
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, TRUE, TRUE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_custom_handler(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=Test Config\n\
@@ -71,12 +79,13 @@ filename=rootfs.ext4\n\
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, TRUE, FALSE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_install_check_hook(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=Test Config\n\
@@ -94,12 +103,13 @@ filename=appfs.ext4";
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, FALSE, TRUE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_install_hook(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=Test Config\n\
@@ -118,12 +128,13 @@ hooks=install";
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, FALSE, TRUE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_bundle_post_hook(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=Test Config\n\
@@ -142,13 +153,13 @@ hooks=post-install";
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, FALSE, TRUE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
 static void install_fixture_set_up_system_conf(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar* pathname = NULL;
+	g_autofree gchar* pathname = NULL;
 	const gchar *cfg_file = "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
@@ -215,8 +226,6 @@ device=/path/to/prebootloader";
 	pathname = write_tmp_file(fixture->tmpdir, "system.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 	r_context_conf()->configpath = g_strdup(pathname);
-
-	g_free(pathname);
 }
 
 static void install_fixture_tear_down(InstallFixture *fixture,
@@ -239,10 +248,10 @@ static void install_test_bootname(InstallFixture *fixture,
 static void install_test_target(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucManifest *rm = NULL;
-	GHashTable *tgrp;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 	GList *selected_images = NULL;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	gboolean result;
 
 
@@ -331,16 +340,14 @@ filename=bootloader.img";
 	g_assert_cmpstr(((RaucImage*)g_list_nth_data(selected_images, 1))->filename, ==, "appfs.ext4");
 	g_assert_cmpstr(((RaucImage*)g_list_nth_data(selected_images, 2))->filename, ==, "demofs.ext4");
 	g_assert_cmpstr(((RaucImage*)g_list_nth_data(selected_images, 3))->filename, ==, "bootloader.img");
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test with image for non-redundant active target slot. */
 static void test_install_determine_target_group_non_redundant(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -356,7 +363,6 @@ device=/dev/null\n\
 
 	sysconfpath = write_tmp_file(tmpdir, "test.conf", system_conf, NULL);
 	g_assert_nonnull(sysconfpath);
-	g_free(tmpdir);
 
 	/* Set up context */
 	r_context_conf()->configpath = sysconfpath;
@@ -370,17 +376,15 @@ device=/dev/null\n\
 
 	/* We must not have any updatable slot detected */
 	g_assert_cmpint(g_hash_table_size(tgrp), ==, 0);
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test a typical asynchronous slot setup (rootfs + rescuefs) with additional
  * childs */
 static void test_install_target_group_async(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -408,7 +412,6 @@ device=/dev/null\n\
 
 	sysconfpath = write_tmp_file(tmpdir, "test.conf", system_conf, NULL);
 	g_assert_nonnull(sysconfpath);
-	g_free(tmpdir);
 
 	/* Set up context */
 	r_context_conf()->configpath = sysconfpath;
@@ -427,16 +430,14 @@ device=/dev/null\n\
 
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "rootfs"))->name, ==, "rootfs.0");
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "appfs"))->name, ==, "appfs.0");
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test a typical synchronous slot setup (rootfs a + b) with appfs childs */
 static void test_install_target_group_sync(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -481,16 +482,14 @@ device=/dev/null\n\
 
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "rootfs"))->name, ==, "rootfs.0");
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "appfs"))->name, ==, "appfs.0");
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test with extra loose (non-booted) groups in parent child relation */
 static void test_install_target_group_loose(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -530,16 +529,14 @@ device=/dev/null\n\
 
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "xloader"))->name, ==, "xloader.0");
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "bootloader"))->name, ==, "bootloader.0");
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test with 3 redundant slots */
 static void test_install_target_group_n_redundant(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -578,21 +575,20 @@ device=/dev/null\n\
 	g_assert_true(g_hash_table_contains(tgrp, "rootfs"));
 
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "rootfs"))->name, ==, "rootfs.0");
-
-	g_hash_table_unref(tgrp);
 }
 
 /* Test image selection, default redundancy setup */
 static void test_install_image_selection(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GBytes *data = NULL;
-	RaucManifest *rm = NULL;
-	GHashTable *tgrp = NULL;
-	GError *error = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
 	GList *selected_images = NULL;
 	RaucImage *image = NULL;
+	gboolean res;
 
 #define MANIFEST2 "\
 [update]\n\
@@ -638,11 +634,13 @@ device=/dev/null\n\
 	r_context();
 
 	data = g_bytes_new_static(MANIFEST2, sizeof(MANIFEST2));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
-	determine_slot_states(&error);
+	res = determine_slot_states(&error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -662,19 +660,18 @@ device=/dev/null\n\
 	image = (RaucImage*) g_list_nth_data(selected_images, 1);
 	g_assert_nonnull(image);
 	g_assert_cmpstr(image->filename, ==, "appfs.img");
-
-	g_hash_table_unref(tgrp);
 }
 
 static void test_install_image_selection_no_matching_slot(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GBytes *data = NULL;
-	RaucManifest *rm = NULL;
-	GHashTable *tgrp = NULL;
-	GError *error = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
 	GList *selected_images = NULL;
+	gboolean res;
 
 #define MANIFEST2 "\
 [update]\n\
@@ -711,11 +708,13 @@ device=/dev/null\n\
 	r_context();
 
 	data = g_bytes_new_static(MANIFEST2, sizeof(MANIFEST2));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
-	determine_slot_states(&error);
+	res = determine_slot_states(&error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -725,19 +724,18 @@ device=/dev/null\n\
 	selected_images = get_install_images(rm, tgrp, &error);
 	g_assert_null(selected_images);
 	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
-
-	g_hash_table_unref(tgrp);
 }
 
 static void test_install_image_readonly(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GBytes *data = NULL;
-	RaucManifest *rm = NULL;
-	GHashTable *tgrp = NULL;
-	GError *error = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
 	GList *selected_images = NULL;
+	gboolean res;
 
 #define MANIFEST "\
 [update]\n\
@@ -771,11 +769,13 @@ readonly=true\n\
 	r_context();
 
 	data = g_bytes_new_static(MANIFEST, sizeof(MANIFEST));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
-	determine_slot_states(&error);
+	res = determine_slot_states(&error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -785,21 +785,20 @@ readonly=true\n\
 	selected_images = get_install_images(rm, tgrp, &error);
 	g_assert_null(selected_images);
 	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
-
-	g_hash_table_unref(tgrp);
 }
 
 
 static void test_install_image_variants(void)
 {
-	gchar *tmpdir = NULL;
-	gchar* sysconfpath = NULL;
-	GBytes *data = NULL;
-	RaucManifest *rm = NULL;
-	GHashTable *tgrp = NULL;
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
 	GList *install_images = NULL;
 	RaucImage *test_img = NULL;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 #define MANIFEST_VARIANT "\
 [update]\n\
@@ -853,22 +852,23 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("system1");
 	r_context();
 
-	determine_slot_states(&error);
+	res = determine_slot_states(&error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
 
 	/* Test with manifest containing default and specific variant */
 	data = g_bytes_new_static(MANIFEST_VARIANT, sizeof(MANIFEST_VARIANT));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 	g_assert_nonnull(rm);
 
 	install_images = get_install_images(rm, tgrp, &error);
 	g_assert_no_error(error);
 	g_assert_nonnull(install_images);
-	g_clear_pointer(&rm, free_manifest);
 
 	g_assert_cmpint(g_list_length(install_images), ==, 1);
 
@@ -876,16 +876,19 @@ device=/dev/null\n\
 	g_assert_nonnull(test_img);
 	g_assert_cmpstr(test_img->variant, ==, "variant-1");
 
+	g_clear_pointer(&rm, free_manifest);
+	g_clear_pointer(&data, g_bytes_unref);
+
 	/* Test with manifest containing only default variant */
 	data = g_bytes_new_static(MANIFEST_DEFAULT_VARIANT, sizeof(MANIFEST_DEFAULT_VARIANT));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 	g_assert_nonnull(rm);
 
 	install_images = get_install_images(rm, tgrp, &error);
 	g_assert_no_error(error);
 	g_assert_nonnull(install_images);
-	g_clear_pointer(&rm, free_manifest);
 
 	g_assert_cmpint(g_list_length(install_images), ==, 1);
 
@@ -893,18 +896,19 @@ device=/dev/null\n\
 	g_assert_nonnull(test_img);
 	g_assert_null(test_img->variant);
 
+	g_clear_pointer(&rm, free_manifest);
+	g_clear_pointer(&data, g_bytes_unref);
+
 	/* Test with manifest containing only non-matching specific variant (must fail) */
 	data = g_bytes_new_static(MANIFEST_OTHER_VARIANT, sizeof(MANIFEST_OTHER_VARIANT));
-	load_manifest_mem(data, &rm, &error);
+	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
+	g_assert_true(res);
 	g_assert_nonnull(rm);
 
 	install_images = get_install_images(rm, tgrp, &error);
 	g_assert_null(install_images);
 	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
-	g_clear_pointer(&rm, free_manifest);
-
-	g_hash_table_unref(tgrp);
 }
 
 static gboolean r_quit(gpointer data)
@@ -940,12 +944,27 @@ static gboolean install_cleanup(gpointer data)
 	return G_SOURCE_REMOVE;
 }
 
+static gint find_str_custom(gconstpointer a, gconstpointer b)
+{
+	return g_strcmp0((const char*) a, (const char *) b);
+}
+
+static gboolean install_args_find_message(RaucInstallArgs *args, const gchar *needle)
+{
+	return g_queue_find_custom(&args->status_messages, needle, find_str_custom) != NULL;
+}
+
 static void install_test_bundle(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlepath, *mountprefix, *slotfile, *testfilepath, *mountdir;
+	InstallData *data = (InstallData*) user_data;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountprefix = NULL;
+	g_autofree gchar *slotfile = NULL;
+	g_autofree gchar *testfilepath = NULL;
+	g_autofree gchar *mountdir = NULL;
 	RaucInstallArgs *args;
-	GError *ierror = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res;
 
 	/* needs to run as root */
@@ -955,14 +974,14 @@ static void install_test_bundle(InstallFixture *fixture,
 	/* Set mount path to current temp dir */
 	mountprefix = g_build_filename(fixture->tmpdir, "mount", NULL);
 	g_assert_nonnull(mountprefix);
-	r_context_conf()->mountprefix = mountprefix;
+	r_context_conf()->mountprefix = g_steal_pointer(&mountprefix);
 	r_context();
 
 	bundlepath = g_build_filename(fixture->tmpdir, "bundle.raucb", NULL);
 	g_assert_nonnull(bundlepath);
 
 	args = install_args_new();
-	args->name = g_strdup(bundlepath);
+	args->name = g_steal_pointer(&bundlepath);
 	args->notify = install_notify;
 	args->cleanup = install_cleanup;
 	res = do_install_bundle(args, &ierror);
@@ -977,19 +996,23 @@ static void install_test_bundle(InstallFixture *fixture,
 	g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
 	g_assert(test_umount(fixture->tmpdir, "mnt"));
 
-	args->status_result = 0;
+	if (data != NULL && data->message_needles != NULL) {
+		const gchar **message_needles = data->message_needles;
+		while (*message_needles != NULL) {
+			g_assert_true(install_args_find_message(args, *message_needles));
+			message_needles++;
+		}
+	}
 
-	g_free(bundlepath);
-	g_free(slotfile);
-	g_free(mountdir);
-	g_free(testfilepath);
+	args->status_result = 0;
 }
 
 static void install_test_bundle_thread(InstallFixture *fixture,
 		gconstpointer user_data)
 {
 	RaucInstallArgs *args = install_args_new();
-	gchar *bundlepath, *mountdir;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountdir = NULL;
 
 	/* needs to run as root */
 	if (!test_running_as_root())
@@ -1012,16 +1035,15 @@ static void install_test_bundle_thread(InstallFixture *fixture,
 	g_assert_true(install_run(args));
 	g_main_loop_run(r_loop);
 	g_clear_pointer(&r_loop, g_main_loop_unref);
-
-	g_free(bundlepath);
 }
 
 static void install_test_bundle_hook_install_check(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlepath, *mountdir;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountdir = NULL;
 	RaucInstallArgs *args;
-	GError *ierror = NULL;
+	g_autoptr(GError) ierror = NULL;
 
 	/* needs to run as root */
 	if (!test_running_as_root())
@@ -1044,17 +1066,18 @@ static void install_test_bundle_hook_install_check(InstallFixture *fixture,
 	g_assert_cmpstr(ierror->message, ==, "Installation error: Bundle rejected: Hook returned: No, I won't install this!");
 
 	args->status_result = 0;
-
-	g_free(bundlepath);
-	g_free(mountdir);
 }
 
 static void install_test_bundle_hook_install(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlepath, *mountdir, *slotfile, *stamppath, *hookfilepath;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountdir = NULL;
+	g_autofree gchar *slotfile = NULL;
+	g_autofree gchar *stamppath = NULL;
+	g_autofree gchar *hookfilepath = NULL;
 	RaucInstallArgs *args;
-	GError *ierror = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
 	/* needs to run as root */
@@ -1085,27 +1108,26 @@ static void install_test_bundle_hook_install(InstallFixture *fixture,
 	g_assert_true(g_file_test(hookfilepath, G_FILE_TEST_IS_REGULAR));
 	g_assert_false(g_file_test(stamppath, G_FILE_TEST_IS_REGULAR));
 	g_assert(test_umount(fixture->tmpdir, "mount"));
-	g_free(hookfilepath);
-	g_free(stamppath);
-	g_free(slotfile);
 
+	g_free(slotfile);
 	slotfile = g_build_filename(fixture->tmpdir, "images/appfs-1", NULL);
+	g_free(stamppath);
 	stamppath = g_build_filename(mountdir, "hook-stamp", NULL);
 	g_assert(test_mount(slotfile, mountdir));
 	g_assert_false(g_file_test(stamppath, G_FILE_TEST_IS_REGULAR));
 	g_assert(test_umount(fixture->tmpdir, "mount"));
-	g_free(stamppath);
-	g_free(slotfile);
 
 	args->status_result = 0;
-
-	g_free(bundlepath);
 }
 
 static void install_test_bundle_hook_post_install(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlepath, *mountdir, *slotfile, *testfilepath, *stamppath;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountdir = NULL;
+	g_autofree gchar *slotfile = NULL;
+	g_autofree gchar *testfilepath = NULL;
+	g_autofree gchar *stamppath = NULL;
 	RaucInstallArgs *args;
 
 	/* needs to run as root */
@@ -1134,30 +1156,27 @@ static void install_test_bundle_hook_post_install(InstallFixture *fixture,
 	g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
 	g_assert(g_file_test(stamppath, G_FILE_TEST_IS_REGULAR));
 	g_assert(test_umount(fixture->tmpdir, "mount"));
-	g_free(stamppath);
-	g_free(slotfile);
-	g_free(testfilepath);
 
+	g_free(slotfile);
 	slotfile = g_build_filename(fixture->tmpdir, "images/appfs-1", NULL);
+	g_free(stamppath);
 	stamppath = g_build_filename(mountdir, "hook-stamp", NULL);
 	g_assert(test_mount(slotfile, mountdir));
 	g_assert(!g_file_test(stamppath, G_FILE_TEST_IS_REGULAR));
 	g_assert(test_umount(fixture->tmpdir, "mount"));
-	g_free(stamppath);
-	g_free(slotfile);
 
 	args->status_result = 0;
-
-	g_free(bundlepath);
 }
 
 /* Test with already mounted slot */
 static void install_test_already_mounted(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-	gchar *bundlepath, *mountprefix, *hookfilepath;
+	g_autofree gchar *bundlepath = NULL;
+	g_autofree gchar *mountprefix = NULL;
+	g_autofree gchar *hookfilepath = NULL;
 	RaucInstallArgs *args;
-	GError *ierror = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res;
 
 	/* needs to run as root */
@@ -1188,8 +1207,6 @@ static void install_test_already_mounted(InstallFixture *fixture,
 	g_assert_true(g_file_test(hookfilepath, G_FILE_TEST_IS_REGULAR));
 
 	args->status_result = 0;
-
-	g_free(bundlepath);
 }
 
 static void install_fixture_set_up_system_user(InstallFixture *fixture,
@@ -1202,22 +1219,17 @@ static void install_fixture_set_up_system_user(InstallFixture *fixture,
 
 int main(int argc, char *argv[])
 {
-	gchar *path;
+	InstallData *install_data;
+	g_autofree gchar *path = NULL;
 	setlocale(LC_ALL, "C");
 
 	path = g_strdup_printf("%s:%s", g_getenv("PATH"), "test/bin");
 	g_setenv("PATH", path, TRUE);
-	g_free(path);
 
 	g_test_init(&argc, &argv, NULL);
 
-	g_test_add("/install/bootname", InstallFixture, NULL,
-			install_fixture_set_up_system_user, install_test_bootname,
-			install_fixture_tear_down);
-
-	g_test_add("/install/target", InstallFixture, NULL,
-			install_fixture_set_up_system_conf, install_test_target,
-			install_fixture_tear_down);
+	r_context_conf()->handlerextra = g_strdup("--dummy1 --dummy2");
+	r_context();
 
 	g_test_add_func("/install/target-group/non-redundant", test_install_determine_target_group_non_redundant);
 
@@ -1237,37 +1249,106 @@ int main(int argc, char *argv[])
 
 	g_test_add_func("/install/image-mapping/variants", test_install_image_variants);
 
-	g_test_add("/install/bundle", InstallFixture, NULL,
-			install_fixture_set_up_bundle, install_test_bundle,
-			install_fixture_tear_down);
+	for (RManifestBundleFormat format = R_MANIFEST_FORMAT_PLAIN; format <= R_MANIFEST_FORMAT_VERITY; format++) {
+		const gchar *format_name = r_manifest_bundle_format_to_str(format);
 
-	g_test_add("/install/bundle/central-status", InstallFixture, NULL,
-			install_fixture_set_up_bundle_central_status, install_test_bundle,
-			install_fixture_tear_down);
+		install_data = memdup((&(InstallData) {
+			.manifest_test_options = {
+			        .format = format,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/bootname/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_system_user, install_test_bootname,
+				install_fixture_tear_down);
 
-	g_test_add("/install/bundle-thread", InstallFixture, NULL,
-			install_fixture_set_up_bundle, install_test_bundle_thread,
-			install_fixture_tear_down);
+		g_test_add(g_strdup_printf("/install/target/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_system_conf, install_test_target,
+				install_fixture_tear_down);
 
-	g_test_add("/install/bundle-custom-handler", InstallFixture, NULL,
-			install_fixture_set_up_bundle_custom_handler, install_test_bundle,
-			install_fixture_tear_down);
+		g_test_add(g_strdup_printf("/install/bundle/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle, install_test_bundle,
+				install_fixture_tear_down);
 
-	g_test_add("/install/bundle-hook/install-check", InstallFixture, NULL,
-			install_fixture_set_up_bundle_install_check_hook, install_test_bundle_hook_install_check,
-			install_fixture_tear_down);
+		g_test_add(g_strdup_printf("/install/bundle/central-status/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_central_status, install_test_bundle,
+				install_fixture_tear_down);
 
-	g_test_add("/install/bundle-hook/slot-install", InstallFixture, NULL,
-			install_fixture_set_up_bundle_install_hook, install_test_bundle_hook_install,
-			install_fixture_tear_down);
+		g_test_add(g_strdup_printf("/install/bundle-thread/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle, install_test_bundle_thread,
+				install_fixture_tear_down);
 
-	g_test_add("/install/bundle-hook/slot-post-install", InstallFixture, NULL,
-			install_fixture_set_up_bundle_post_hook, install_test_bundle_hook_post_install,
-			install_fixture_tear_down);
+		install_data = memdup((&(InstallData) {
+			.message_needles = (const gchar *[]) {
+			        "Checking and mounting bundle...",
+			        "Debug: --dummy1 --dummy2",
+			        "Handler status: [STARTED]",
+			        "Bootloader status: [DONE]",
+			        "Handler status: [DONE]",
+			        NULL,
+			},
+			.manifest_test_options = {
+			        .custom_handler = TRUE,
+			        .hooks = FALSE,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/bundle-custom-handler/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_custom_handler, install_test_bundle,
+				install_fixture_tear_down);
 
-	g_test_add("/install/already-mounted", InstallFixture, NULL,
-			install_fixture_set_up_bundle_already_mounted, install_test_already_mounted,
-			install_fixture_tear_down);
+		install_data = memdup((&(InstallData) {
+			.message_needles = NULL,
+			.manifest_test_options = {
+			        .custom_handler = FALSE,
+			        .hooks = TRUE,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/bundle-hook/install-check/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_install_check_hook, install_test_bundle_hook_install_check,
+				install_fixture_tear_down);
+
+		install_data = memdup((&(InstallData) {
+			.message_needles = NULL,
+			.manifest_test_options = {
+			        .custom_handler = FALSE,
+			        .hooks = TRUE,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/bundle-hook/slot-install/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_install_hook, install_test_bundle_hook_install,
+				install_fixture_tear_down);
+
+		install_data = memdup((&(InstallData) {
+			.message_needles = NULL,
+			.manifest_test_options = {
+			        .custom_handler = FALSE,
+			        .hooks = TRUE,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/bundle-hook/slot-post-install/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_post_hook, install_test_bundle_hook_post_install,
+				install_fixture_tear_down);
+
+		install_data = memdup((&(InstallData) {
+			.message_needles = NULL,
+			.manifest_test_options = {
+			        .custom_handler = TRUE,
+			        .hooks = TRUE,
+			},
+		}));
+		g_test_add(g_strdup_printf("/install/already-mounted/%s", format_name),
+				InstallFixture, install_data,
+				install_fixture_set_up_bundle_already_mounted, install_test_already_mounted,
+				install_fixture_tear_down);
+	}
 
 	return g_test_run();
 }
