@@ -29,7 +29,11 @@ static void service_install_fixture_set_up(ServiceFixture *fixture, gconstpointe
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, FALSE, FALSE);
+	fixture_helper_set_up_bundle(fixture->tmpdir, NULL,
+			&(ManifestTestOptions) {
+		.custom_handler = FALSE,
+		.hooks = FALSE,
+	});
 
 	/* Write a D-Bus service file with current tmpdir */
 	write_tmp_file(fixture->tmpdir, "de.pengutronix.rauc.service", g_strdup_printf("\
@@ -175,8 +179,8 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  20, "Verifying signature", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Verifying signature done.", 3));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Checking bundle done.", 2));
-	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Loading manifest file", 2));
-	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  60, "Loading manifest file done.", 2));
+	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  40, "Checking manifest contents", 2));
+	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  60, "Checking manifest contents done.", 2));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  60, "Determining target install group", 2));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  80, "Determining target install group done.", 2));
 	g_queue_push_tail(args, (gpointer*)g_variant_new("(isi)",  80, "Updating slots", 2));
@@ -193,8 +197,9 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 
 	installer = r_installer_proxy_new_for_bus_sync(G_BUS_TYPE_SESSION,
 			G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
-			"de.pengutronix.rauc", "/", NULL, NULL);
+			"de.pengutronix.rauc", "/", NULL, &error);
 
+	g_assert_no_error(error);
 	g_assert_nonnull(installer);
 
 	/* connect signals to test callbacks */
@@ -245,7 +250,7 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 
 	g_main_loop_run(testloop);
 
-	g_clear_pointer(&installer, g_object_unref);
+	g_clear_object(&installer);
 }
 
 static void service_test_install_bundle(ServiceFixture *fixture, gconstpointer user_data)
@@ -259,11 +264,18 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 	gboolean ret = FALSE;
 	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 
+	if (!ENABLE_SERVICE) {
+		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
+		return;
+	}
+
 	testloop = g_main_loop_new(NULL, FALSE);
 
 	installer = r_installer_proxy_new_for_bus_sync(G_BUS_TYPE_SESSION,
 			G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
-			"de.pengutronix.rauc", "/", NULL, NULL);
+			"de.pengutronix.rauc", "/", NULL, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(installer);
 
 	g_assert_cmpint(g_signal_connect(installer, "completed",
 			G_CALLBACK(on_installer_completed_failed), NULL), !=, 0);
@@ -313,7 +325,7 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 
 	g_main_loop_run(testloop);
 
-	g_clear_pointer(&installer, g_object_unref);
+	g_clear_object(&installer);
 }
 
 static void service_test_install_deprecated(ServiceFixture *fixture, gconstpointer user_data)
@@ -326,18 +338,23 @@ static void service_test_info(ServiceFixture *fixture, gconstpointer user_data)
 	GError *error = NULL;
 	gchar *compatible;
 	gchar *version;
+	g_autofree gchar *bundlepath = NULL;
 
 	if (!ENABLE_SERVICE) {
 		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
 		return;
 	}
 
+	bundlepath = g_build_filename(fixture->tmpdir, "good-bundle.raucb", NULL);
+	g_assert_true(test_copy_file("test", "good-bundle.raucb", fixture->tmpdir, "good-bundle.raucb"));
+
 	installer = r_installer_proxy_new_for_bus_sync(G_BUS_TYPE_SESSION,
 			G_DBUS_PROXY_FLAGS_NONE,
 			"de.pengutronix.rauc",
 			"/",
 			NULL,
-			NULL);
+			&error);
+	g_assert_no_error(error);
 
 	if (installer == NULL) {
 		g_error("failed to install proxy");
@@ -345,7 +362,7 @@ static void service_test_info(ServiceFixture *fixture, gconstpointer user_data)
 	}
 
 	r_installer_call_info_sync(installer,
-			"test/good-bundle.raucb",
+			bundlepath,
 			&compatible,
 			&version,
 			NULL,
@@ -355,7 +372,7 @@ static void service_test_info(ServiceFixture *fixture, gconstpointer user_data)
 	g_assert_cmpstr(version, ==, "2011.03-2");
 
 out:
-	g_clear_pointer(&installer, g_object_unref);
+	g_clear_object(&installer);
 	g_free(compatible);
 	g_free(version);
 }
@@ -375,7 +392,8 @@ static void service_test_slot_status(ServiceFixture *fixture, gconstpointer user
 			"de.pengutronix.rauc",
 			"/",
 			NULL,
-			NULL);
+			&error);
+	g_assert_no_error(error);
 
 	if (installer == NULL) {
 		g_error("failed to install proxy");
@@ -391,7 +409,7 @@ static void service_test_slot_status(ServiceFixture *fixture, gconstpointer user
 	g_assert_cmpint(g_variant_n_children(slot_status_array), ==, 6);
 
 out:
-	g_clear_pointer(&installer, g_object_unref);
+	g_clear_object(&installer);
 	g_variant_unref(slot_status_array);
 }
 
