@@ -1,6 +1,8 @@
 #include <stdio.h>
 #include <locale.h>
 #include <glib.h>
+#include <glib/gstdio.h>
+#include <fcntl.h>
 
 #include <context.h>
 #include <utils.h>
@@ -47,25 +49,33 @@ static void signature_tear_down(SignatureFixture *fixture,
 	r_context_clean();
 }
 
-static void signature_sign(SignatureFixture *fixture,
+static void signature_sign_detached(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
 	gchar *certpath = g_strdup("test/openssl-ca/rel/release-1.cert.pem");
 	gchar *keypath = g_strdup("test/openssl-ca/rel/private/release-1.pem");
+	gboolean detached = FALSE;
 
 	// Test valid signing
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			certpath,
 			keypath,
 			NULL,
 			&fixture->error);
+	g_assert_no_error(fixture->error);
 	g_assert_nonnull(fixture->sig);
 	g_assert_null(fixture->error);
 
+	g_assert_true(cms_is_detached(fixture->sig, &detached, &fixture->error));
+	g_assert_no_error(fixture->error);
+	g_assert_true(detached);
+
 	g_bytes_unref(fixture->sig);
 
 	// Test signing fails with invalid key
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			certpath,
 			"test/random.dat",
 			NULL,
@@ -77,6 +87,54 @@ static void signature_sign(SignatureFixture *fixture,
 
 	// Test signing fails with invalid cert
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
+			"test/random.dat",
+			keypath,
+			NULL,
+			&fixture->error);
+	g_assert_null(fixture->sig);
+	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_PARSE_ERROR);
+}
+
+static void signature_sign_inline(SignatureFixture *fixture,
+		gconstpointer user_data)
+{
+	gchar *certpath = g_strdup("test/openssl-ca/rel/release-1.cert.pem");
+	gchar *keypath = g_strdup("test/openssl-ca/rel/private/release-1.pem");
+	gboolean detached = TRUE;
+
+	// Test valid signing
+	fixture->sig = cms_sign(fixture->content,
+			FALSE,
+			certpath,
+			keypath,
+			NULL,
+			&fixture->error);
+	g_assert_no_error(fixture->error);
+	g_assert_nonnull(fixture->sig);
+	g_assert_null(fixture->error);
+
+	g_assert_true(cms_is_detached(fixture->sig, &detached, &fixture->error));
+	g_assert_no_error(fixture->error);
+	g_assert_false(detached);
+
+	g_bytes_unref(fixture->sig);
+
+	// Test signing fails with invalid key
+	fixture->sig = cms_sign(fixture->content,
+			FALSE,
+			certpath,
+			"test/random.dat",
+			NULL,
+			&fixture->error);
+	g_assert_null(fixture->sig);
+	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_PARSE_ERROR);
+
+	g_clear_error(&fixture->error);
+
+	// Test signing fails with invalid cert
+	fixture->sig = cms_sign(fixture->content,
+			FALSE,
 			"test/random.dat",
 			keypath,
 			NULL,
@@ -128,15 +186,21 @@ static void signature_sign_file(SignatureFixture *fixture,
 static void signature_verify_valid(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
+	gboolean detached = FALSE;
 	gboolean res;
 
 	fixture->sig = read_file("test/openssl-ca/manifest-r1.sig", NULL);
 	g_assert_nonnull(fixture->sig);
 
-	res = cms_verify(fixture->content,
+	g_assert_true(cms_is_detached(fixture->sig, &detached, &fixture->error));
+	g_assert_no_error(fixture->error);
+	g_assert_true(detached);
+
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			fixture->store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -146,14 +210,23 @@ static void signature_verify_valid(SignatureFixture *fixture,
 static void signature_verify_invalid(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
+	gboolean detached = TRUE;
+
 	fixture->sig = read_file("test/random.dat", NULL);
 	g_assert_nonnull(fixture->sig);
 
 	// Test against invalid signature
-	g_assert_false(cms_verify(fixture->content,
+	g_assert_false(cms_is_detached(fixture->sig, &detached, &fixture->error));
+	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_PARSE);
+	g_assert_true(detached); /* should not be touched in the error case */
+
+	g_clear_error(&fixture->error);
+
+	g_assert_false(cms_verify_bytes(fixture->content,
 			fixture->sig,
 			fixture->store,
 			&fixture->cms,
+			NULL,
 			&fixture->error));
 	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_PARSE);
 	g_assert_null(fixture->cms);
@@ -162,17 +235,21 @@ static void signature_verify_invalid(SignatureFixture *fixture,
 static void signature_verify_file(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
+	gint fd;
 	gboolean res;
 	fixture->sig = read_file("test/openssl-ca/manifest-r1.sig", NULL);
 	g_assert_nonnull(fixture->sig);
 
 	// Test valid manifest
-	res = cms_verify_file("test/openssl-ca/manifest",
+	fd = g_open("test/openssl-ca/manifest", O_RDONLY|O_CLOEXEC, 0);
+	g_assert_cmpint(fd, >=, 0);
+	res = cms_verify_fd(fd,
 			fixture->sig,
 			0,
 			fixture->store,
 			&fixture->cms,
 			&fixture->error);
+	g_close(fd, NULL);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
 	g_assert_nonnull(fixture->cms);
@@ -180,40 +257,94 @@ static void signature_verify_file(SignatureFixture *fixture,
 	g_clear_pointer(&fixture->cms, CMS_ContentInfo_free);
 
 	// Test valid manifest with invalid size limit
-	g_assert_false(cms_verify_file("test/openssl-ca/manifest",
+	fd = g_open("test/openssl-ca/manifest", O_RDONLY|O_CLOEXEC, 0);
+	g_assert_cmpint(fd, >=, 0);
+	res = cms_verify_fd(fd,
 			fixture->sig,
 			42,
 			fixture->store,
 			&fixture->cms,
-			&fixture->error));
+			&fixture->error);
+	g_close(fd, NULL);
+	g_assert_false(res);
 	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_assert_null(fixture->cms);
 
 	g_clear_error(&fixture->error);
-
-	// Test non-existing file
-	g_assert_false(cms_verify_file("path/to/nonexisting/file",
-			fixture->sig,
-			0,
-			fixture->store,
-			&fixture->cms,
-			&fixture->error));
-	g_assert_error(fixture->error, G_FILE_ERROR, G_FILE_ERROR_NOENT);
-	g_assert_null(fixture->cms);
 }
 
-static void signature_loopback(SignatureFixture *fixture,
+static void signature_loopback_detached(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
+	gboolean res;
+
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/rel/release-1.cert.pem",
 			"test/openssl-ca/rel/private/release-1.pem",
 			NULL,
-			NULL);
+			&fixture->error);
+	g_assert_no_error(fixture->error);
 	g_assert_nonnull(fixture->sig);
-	g_assert_true(cms_verify(fixture->content, fixture->sig, fixture->store, NULL, NULL));
+
+	res = cms_verify_bytes(fixture->content,
+			fixture->sig,
+			fixture->store,
+			NULL,
+			NULL,
+			&fixture->error);
+	g_assert_no_error(fixture->error);
+	g_assert_true(res);
+
 	((char *)g_bytes_get_data(fixture->content, NULL))[0] = 0x00;
-	g_assert_false(cms_verify(fixture->content, fixture->sig, fixture->store, NULL, NULL));
+	res = cms_verify_bytes(fixture->content,
+			fixture->sig,
+			fixture->store,
+			NULL,
+			NULL,
+			&fixture->error);
+	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
+	g_assert_false(res);
+}
+
+static void signature_loopback_inline(SignatureFixture *fixture,
+		gconstpointer user_data)
+{
+	gboolean res;
+	GBytes *manifest = NULL;
+
+	fixture->sig = cms_sign(fixture->content,
+			FALSE,
+			"test/openssl-ca/rel/release-1.cert.pem",
+			"test/openssl-ca/rel/private/release-1.pem",
+			NULL,
+			&fixture->error);
+	g_assert_no_error(fixture->error);
+	g_assert_nonnull(fixture->sig);
+
+	res = cms_verify_bytes(NULL,
+			fixture->sig,
+			fixture->store,
+			NULL,
+			&manifest,
+			&fixture->error);
+	g_assert_no_error(fixture->error);
+	g_assert_true(res);
+	g_assert_nonnull(manifest);
+	g_assert_true(g_bytes_equal(fixture->content, manifest));
+
+	g_clear_pointer(&manifest, g_bytes_unref);
+
+	((char *)g_bytes_get_data(fixture->sig, NULL))[0x10] = 0x00;
+	res = cms_verify_bytes(NULL,
+			fixture->sig,
+			fixture->store,
+			NULL,
+			&manifest,
+			&fixture->error);
+	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_PARSE);
+	g_assert_false(res);
+	g_assert_null(manifest);
 }
 
 static void signature_get_cert_chain(SignatureFixture *fixture,
@@ -223,10 +354,11 @@ static void signature_get_cert_chain(SignatureFixture *fixture,
 	fixture->sig = read_file("test/openssl-ca/manifest-r1.sig", NULL);
 	g_assert_nonnull(fixture->sig);
 
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			fixture->store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -254,6 +386,7 @@ static void signature_selfsigned(SignatureFixture *fixture,
 	g_assert_true(X509_STORE_load_locations(root_store, "test/openssl-ca/root/ca.cert.pem", NULL));
 
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/root/ca.cert.pem",
 			"test/openssl-ca/root/private/ca.key.pem",
 			NULL,
@@ -263,10 +396,11 @@ static void signature_selfsigned(SignatureFixture *fixture,
 
 	g_clear_error(&fixture->error);
 
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			root_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -298,6 +432,7 @@ static void signature_intermediate(SignatureFixture *fixture,
 	g_assert_true(X509_STORE_load_locations(prov_store, "test/openssl-ca/provisioning-ca.pem", NULL));
 
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/rel/release-1.cert.pem",
 			"test/openssl-ca/rel/private/release-1.pem",
 			NULL,
@@ -306,10 +441,11 @@ static void signature_intermediate(SignatureFixture *fixture,
 	g_assert_nonnull(fixture->sig);
 
 	/* Without explicit intermediate certificate, this must fail */
-	g_assert_false(cms_verify(fixture->content,
+	g_assert_false(cms_verify_bytes(fixture->content,
 			fixture->sig,
 			prov_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error));
 	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_assert_null(fixture->cms);
@@ -323,6 +459,7 @@ static void signature_intermediate(SignatureFixture *fixture,
 	g_ptr_array_add(interfiles, NULL);
 
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/rel/release-1.cert.pem",
 			"test/openssl-ca/rel/private/release-1.pem",
 			(gchar**) g_ptr_array_free(interfiles, FALSE),
@@ -330,10 +467,11 @@ static void signature_intermediate(SignatureFixture *fixture,
 	g_assert_nonnull(fixture->sig);
 
 	/* With intermediate certificate, this must succeed */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			prov_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -355,6 +493,7 @@ static void signature_intermediate(SignatureFixture *fixture,
 static void signature_intermediate_file(SignatureFixture *fixture,
 		gconstpointer user_data)
 {
+	gint fd;
 	gboolean res;
 	GPtrArray *interfiles = NULL;
 	X509_STORE *prov_store = X509_STORE_new();
@@ -370,12 +509,16 @@ static void signature_intermediate_file(SignatureFixture *fixture,
 	g_assert_nonnull(fixture->sig);
 
 	/* Without explicit intermediate certificate, this must fail */
-	g_assert_false(cms_verify_file("test/openssl-ca/manifest",
+	fd = g_open("test/openssl-ca/manifest", O_RDONLY|O_CLOEXEC, 0);
+	g_assert_cmpint(fd, >=, 0);
+	res = cms_verify_fd(fd,
 			fixture->sig,
 			0,
 			prov_store,
 			&fixture->cms,
-			&fixture->error));
+			&fixture->error);
+	g_close(fd, NULL);
+	g_assert_false(res);
 	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_assert_null(fixture->cms);
 
@@ -395,12 +538,15 @@ static void signature_intermediate_file(SignatureFixture *fixture,
 	g_assert_nonnull(fixture->sig);
 
 	/* With intermediate certificate, this must succeed */
-	res = cms_verify_file("test/openssl-ca/manifest",
+	fd = g_open("test/openssl-ca/manifest", O_RDONLY|O_CLOEXEC, 0);
+	g_assert_cmpint(fd, >=, 0);
+	res = cms_verify_fd(fd,
 			fixture->sig,
 			0,
 			fixture->store,
 			&fixture->cms,
 			&fixture->error);
+	g_close(fd, NULL);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
 	g_assert_nonnull(fixture->cms);
@@ -428,6 +574,7 @@ static void signature_cmsverify_path(SignatureFixture *fixture,
 
 	/* Sign with "A" key and cert */
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/dir/a.cert.pem",
 			"test/openssl-ca/dir/private/a.key.pem",
 			NULL,
@@ -438,10 +585,11 @@ static void signature_cmsverify_path(SignatureFixture *fixture,
 	g_clear_error(&fixture->error);
 
 	/* Verify against "A" cert */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			a_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -458,6 +606,7 @@ static void signature_cmsverify_dir_combined(SignatureFixture *fixture,
 
 	/* Sign with "A" key and cert */
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/dir/a.cert.pem",
 			"test/openssl-ca/dir/private/a.key.pem",
 			NULL,
@@ -467,10 +616,11 @@ static void signature_cmsverify_dir_combined(SignatureFixture *fixture,
 	g_clear_error(&fixture->error);
 
 	/* Verify against certs stored in combined directory (A+B) */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			ab_dir_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -491,6 +641,7 @@ static void signature_cmsverify_dir_single_fail(SignatureFixture *fixture,
 
 	/* Sign with "B" key and cert */
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/dir/b.cert.pem",
 			"test/openssl-ca/dir/private/b.key.pem",
 			NULL,
@@ -500,10 +651,11 @@ static void signature_cmsverify_dir_single_fail(SignatureFixture *fixture,
 	g_clear_error(&fixture->error);
 
 	/* Verify against certs stored in combined directory (A+B) */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			ab_dir_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -511,10 +663,11 @@ static void signature_cmsverify_dir_single_fail(SignatureFixture *fixture,
 
 	/* Verify failure against certs stored in "A" only directory */
 	g_clear_pointer(&fixture->cms, CMS_ContentInfo_free);
-	g_assert_false(cms_verify(fixture->content,
+	g_assert_false(cms_verify_bytes(fixture->content,
 			fixture->sig,
 			a_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error));
 	g_assert_error(fixture->error, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 }
@@ -529,6 +682,7 @@ static void signature_cmsverify_pathdir_dir(SignatureFixture *fixture,
 
 	/* Sign with "A" key and cert */
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/dir/a.cert.pem",
 			"test/openssl-ca/dir/private/a.key.pem",
 			NULL,
@@ -538,10 +692,11 @@ static void signature_cmsverify_pathdir_dir(SignatureFixture *fixture,
 	g_clear_error(&fixture->error);
 
 	/* Verify against certs stored in directory(A) + path(B) */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			a_dir_b_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -558,6 +713,7 @@ static void signature_cmsverify_pathdir_path(SignatureFixture *fixture,
 
 	/* Sign with "B" key and cert */
 	fixture->sig = cms_sign(fixture->content,
+			TRUE,
 			"test/openssl-ca/dir/b.cert.pem",
 			"test/openssl-ca/dir/private/b.key.pem",
 			NULL,
@@ -568,10 +724,11 @@ static void signature_cmsverify_pathdir_path(SignatureFixture *fixture,
 	g_clear_error(&fixture->error);
 
 	/* Verify against certs stored in directory(A) + path(B) */
-	res = cms_verify(fixture->content,
+	res = cms_verify_bytes(fixture->content,
 			fixture->sig,
 			a_dir_b_store,
 			&fixture->cms,
+			NULL,
 			&fixture->error);
 	g_assert_no_error(fixture->error);
 	g_assert_true(res);
@@ -591,12 +748,14 @@ int main(int argc, char *argv[])
 
 	g_test_init(&argc, &argv, NULL);
 
-	g_test_add("/signature/sign", SignatureFixture, NULL, signature_set_up, signature_sign, signature_tear_down);
+	g_test_add("/signature/sign_detached", SignatureFixture, NULL, signature_set_up, signature_sign_detached, signature_tear_down);
+	g_test_add("/signature/sign_inline", SignatureFixture, NULL, signature_set_up, signature_sign_inline, signature_tear_down);
 	g_test_add("/signature/sign_file", SignatureFixture, NULL, signature_set_up, signature_sign_file, signature_tear_down);
 	g_test_add("/signature/verify_valid", SignatureFixture, NULL, signature_set_up, signature_verify_valid, signature_tear_down);
 	g_test_add("/signature/verify_invalid", SignatureFixture, NULL, signature_set_up, signature_verify_invalid, signature_tear_down);
 	g_test_add("/signature/verify_file", SignatureFixture, NULL, signature_set_up, signature_verify_file, signature_tear_down);
-	g_test_add("/signature/loopback", SignatureFixture, NULL, signature_set_up, signature_loopback, signature_tear_down);
+	g_test_add("/signature/loopback_detached", SignatureFixture, NULL, signature_set_up, signature_loopback_detached, signature_tear_down);
+	g_test_add("/signature/loopback_inline", SignatureFixture, NULL, signature_set_up, signature_loopback_inline, signature_tear_down);
 	g_test_add("/signature/get_cert_chain", SignatureFixture, NULL, signature_set_up, signature_get_cert_chain, signature_tear_down);
 	g_test_add("/signature/selfsigned", SignatureFixture, NULL, signature_set_up, signature_selfsigned, signature_tear_down);
 	g_test_add("/signature/intermediate", SignatureFixture, NULL, signature_set_up, signature_intermediate, signature_tear_down);
